#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import copy
from unittest.mock import MagicMock, ANY

from gns3.compute_manager import ComputeManager
from gns3.compute import Compute


def test_getCompute():
    cm = ComputeManager()
    compute = cm.getCompute("local")
    assert cm.getCompute("local") == compute


def test_getComputeRemote1X():
    """
    It should convert old server id
    """
    cm = ComputeManager()
    cm.computeDataReceivedCallback({
        "compute_id": "aaecb19d-5a32-4c18-ac61-f54f92a0f594",
        "name": "Test Compute",
        "connected": True,
        "protocol": "http",
        "host": "test.org",
        "port": 3080,
        "user": None,
        "cpu_usage_percent": None,
        "memory_usage_percent": None,
        "disk_usage_percent": None,
        "capabilities": {"test": "a"}
    })

    compute = cm.getCompute("aaecb19d-5a32-4c18-ac61-f54f92a0f594")
    assert cm.getCompute("http://test.org:3080") == compute


def test_deleteCompute(controller):
    callback_delete = MagicMock()
    cm = ComputeManager()
    cm.deleted_signal.connect(callback_delete)
    compute = cm.getCompute("test")
    assert cm.getCompute("test") == compute
    cm.deleteCompute("test")
    assert "test" not in cm._computes
    assert callback_delete.called
    controller._http_client.sendRequest.assert_called_with("DELETE", "/computes/test", None)


def test_listComputesCallback():
    callback = MagicMock()
    cm = ComputeManager()
    cm.created_signal.connect(callback)
    cm._listComputesCallback([
        {
            "compute_id": "local",
            "name": "Local server",
            "connected": False,
            "protocol": "http",
            "host": "localhost",
            "port": 3080,
            "user": None,
            "cpu_usage_percent": None,
            "memory_usage_percent": None,
            "disk_usage_percent": None,
            "capabilities": {"test": "a"}
        }
    ])
    assert cm._computes["local"].name() == "Local server"
    assert callback.called


def test_computeDataReceivedCallback():
    callback_create = MagicMock()
    callback_update = MagicMock()
    cm = ComputeManager()
    cm.created_signal.connect(callback_create)
    cm.updated_signal.connect(callback_update)
    cm.computeDataReceivedCallback({
        "compute_id": "test",
        "name": "Test server",
        "connected": False,
        "protocol": "http",
        "host": "test.org",
        "port": 3080,
        "user": None,
        "cpu_usage_percent": None,
        "memory_usage_percent": None,
        "disk_usage_percent": None,
        "capabilities": {"test": "a"}
    })
    assert cm._computes["test"].name() == "Test server"
    assert cm._computes["test"].protocol() == "http"
    assert cm._computes["test"].host() == "test.org"
    assert cm._computes["test"].port() == 3080
    assert cm._computes["test"].user() is None
    assert cm._computes["test"].capabilities() == {"test": "a"}
    assert cm._computes["test"].connected() is False
    assert callback_create.called
    assert not callback_update.called

    # Data should be update
    cm.computeDataReceivedCallback({
        "compute_id": "test",
        "name": "Test Compute",
        "connected": True,
        "protocol": "http",
        "host": "test.org",
        "port": 3080,
        "user": None,
        "cpu_usage_percent": None,
        "memory_usage_percent": None,
        "disk_usage_percent": None,
        "capabilities": {"test": "a"}
    })
    assert cm._computes["test"].name() == "Test Compute"
    assert cm._computes["test"].connected()
    assert callback_update.called


def test_updateList_deleted(controller):
    cm = ComputeManager()
    computes = []
    computes.append(cm.getCompute("test1"))
    computes.append(cm.getCompute("test2"))
    # This server new to be deleted because exist in compute manager
    # but not in setting list
    cm.getCompute("test3")
    cm.updateList(computes)
    assert "test1" in cm._computes
    assert "test2" in cm._computes
    assert "test3" not in cm._computes


def test_updateList_updated(controller):
    cm = ComputeManager()
    computes = []
    compute = copy.copy(cm.getCompute("test1"))
    computes.append(compute)
    compute.setName("TEST2")
    cm.updateList(computes)
    assert cm._computes["test1"].name() == "TEST2"
    controller._http_client.sendRequest.assert_called_with("PUT", "/computes/test1", None, body=compute.__json__())


def test_updateList_added(controller):
    cm = ComputeManager()
    computes = []
    compute = Compute()
    computes.append(compute)
    controller._http_client = MagicMock()
    cm.updateList(computes)
    assert compute.id() in cm._computes
    controller._http_client.sendRequest.assert_called_with("POST", "/computes", callback=ANY, body=compute.__json__(), params={'connect': True})


def test_updateList_no_change(controller):
    cm = ComputeManager()
    computes = []
    compute = copy.copy(cm.getCompute("test1"))
    computes.append(compute)
    controller._http_client = MagicMock()
    cm.updateList(computes)
    assert not controller._http_client.sendRequest.called



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import sys

from unittest.mock import MagicMock


@pytest.fixture
def real_main_window():
    from gns3.main_window import MainWindow
    main_window = MainWindow()
    return main_window


# Windows GUI doesn't support a local server starting with v3.0
@pytest.mark.skipif(sys.platform.startswith('win') is True, reason='Not for windows')
def test_main_window_settings_changed_signal(real_main_window):
    settings = real_main_window.settings()
    signal = MagicMock()
    real_main_window.settings_updated_signal = signal
    real_main_window.setSettings(settings)
    assert signal.emit.called


# Windows GUI doesn't support a local server starting with v3.0
@pytest.mark.skipif(sys.platform.startswith('win') is True, reason='Not for windows')
def test_main_window_settings_changed_slot(real_main_window):
    instance = MagicMock()
    manager = MagicMock()
    manager.instance.return_value = instance
    real_main_window._appliance_manager = manager
    real_main_window.settingsChangedSlot()
    assert instance.refresh.called



#!/usr/bin/env python
#
# Copyright (C) 2017 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from .qt import QtWidgets, QtGui, qslot


class StatusBarHandler(logging.StreamHandler):

    def __init__(self, widget):
        super().__init__()
        self._widget = widget
        self.setLevel(logging.WARNING)

    def emit(self, record):
        if record.levelno > logging.WARNING:
            self._widget.addError()
        else:
            self._widget.addWarning()


class StatusBar(QtWidgets.QStatusBar):
    """
    The status bar
    """

    def __init__(self, parent, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
        self._parent = parent
        self._errors = 0
        self._warnings = 0

        self._errors_button = QtWidgets.QPushButton()
        self._errors_button.setFlat(True)
        self._errors_button.setIcon(QtGui.QIcon(":/icons/warning.svg"))
        self._errors_button.clicked.connect(self._errorButtonPushedSlot)
        self.addPermanentWidget(self._errors_button)

        self._refresh()

    def addError(self):
        """
        Increment error count
        """
        self._errors += 1
        self._refresh()

    def addWarning(self):
        """
        Increment warning count
        """
        self._warnings += 1
        self._refresh()

    def _refresh(self):
        if self._errors == 0 and self._warnings == 0:
            self._errors_button.setText("")
            self._errors_button.hide()
            return
        self._errors_button.show()
        text = ""
        if self._errors == 1:
            text += "1 error"
        elif self._errors > 1:
            text += "{} errors".format(self._errors)
        if self._warnings == 1:
            text += " 1 warning"
        elif self._warnings > 1:
            text += " {} warnings".format(self._warnings)
        self._errors_button.setText(text)

    @qslot
    def _errorButtonPushedSlot(self, *args, **kwargs):
        self._parent.uiConsoleDockWidget.toggleViewAction().trigger()
        self._warnings = 0
        self._errors = 0
        self._refresh()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import shutil

from gns3.qt import QtCore, QtWidgets, QtNetwork
from gns3.controller import Controller
from gns3.local_server import LocalServer

from ..settings import DEFAULT_CONTROLLER_HOST
from ..ui.setup_wizard_ui import Ui_SetupWizard


import logging
log = logging.getLogger(__name__)


class SetupWizard(QtWidgets.QWizard, Ui_SetupWizard):

    """
    Base class for VM wizard.
    """

    def __init__(self, parent):

        super().__init__(parent)
        self.setupUi(self)
        self.adjustSize()

        self.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        if sys.platform.startswith("darwin"):
            # we want to see the cancel button on OSX
            self.setOptions(QtWidgets.QWizard.NoDefaultButton)

        self.uiLocalServerToolButton.clicked.connect(self._localServerBrowserSlot)
        settings = parent.settings()
        self.uiShowCheckBox.setChecked(settings["hide_setup_wizard"])

        # Mandatory fields
        self.uiLocalControllerWizardPage.registerField("path*", self.uiLocalServerPathLineEdit)

        # load all available addresses
        for address in QtNetwork.QNetworkInterface.allAddresses():
            if address.protocol() in [QtNetwork.QAbstractSocket.IPv4Protocol, QtNetwork.QAbstractSocket.IPv6Protocol]:
                address_string = address.toString()
                if address_string.startswith("169.254") or address_string.startswith("fe80"):
                    # ignore link-local addresses, could not use https://doc.qt.io/qt-5/qhostaddress.html#isLinkLocal
                    # because it was introduced in Qt 5.11
                    continue
                self.uiLocalServerHostComboBox.addItem(address_string, address_string)

        self.uiLocalServerHostComboBox.addItem("localhost", "localhost")  # local host
        self.uiLocalServerHostComboBox.addItem("::", "::")  # all IPv6 addresses
        self.uiLocalServerHostComboBox.addItem("0.0.0.0", "0.0.0.0")  # all IPv4 addresses

        if sys.platform.startswith("linux"):
            # only support local controller on Linux
            self.uiLocalControllerRadioButton.setChecked(True)
        else:
            self.uiLocalControllerRadioButton.setEnabled(False)
            self.uiRemoteControllerRadioButton.setChecked(True)

    def _localServerBrowserSlot(self):
        """
        Slot to open a file browser and select a local server.
        """

        filter = ""
        if sys.platform.startswith("win"):
            filter = "Executable (*.exe);;All files (*.*)"
        server_path = shutil.which("gns3server")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select the local server", server_path, filter)
        if not path:
            return

        self.uiLocalServerPathLineEdit.setText(path)

    def _setPreferencesPane(self, dialog, name):
        """
        Finds the first child of the QTreeWidgetItem name.

        :param dialog: PreferencesDialog instance
        :param name: QTreeWidgetItem name

        :returns: current QWidget
        """

        pane = dialog.uiTreeWidget.findItems(name, QtCore.Qt.MatchFixedString)[0]
        child_pane = pane.child(0)
        dialog.uiTreeWidget.setCurrentItem(child_pane)
        return dialog.uiStackedWidget.currentWidget()

    def initializePage(self, page_id):
        """
        Initialize Wizard pages.

        :param page_id: page identifier
        """

        super().initializePage(page_id)
        if self.page(page_id) == self.uiControllerWizardPage:
            Controller.instance().setDisplayError(False)
        elif self.page(page_id) == self.uiLocalControllerWizardPage:
            local_server_settings = LocalServer.instance().localServerSettings()
            self.uiLocalServerPathLineEdit.setText(local_server_settings["path"])
            index = self.uiLocalServerHostComboBox.findData(local_server_settings["host"])
            if index != -1:
                self.uiLocalServerHostComboBox.setCurrentIndex(index)
            else:
                index = self.uiLocalServerHostComboBox.findText(DEFAULT_CONTROLLER_HOST)
                if index != -1:
                    self.uiLocalServerHostComboBox.setCurrentIndex(index)

            self.uiLocalServerPortSpinBox.setValue(local_server_settings["port"])

        elif self.page(page_id) == self.uiRemoteControllerWizardPage:
            local_server_settings = LocalServer.instance().localServerSettings()
            if local_server_settings["host"] is None:
                self.uiRemoteMainServerHostLineEdit.setText(DEFAULT_CONTROLLER_HOST)
                self.uiRemoteMainServerUserLineEdit.setText("")
                self.uiRemoteMainServerPasswordLineEdit.setText("")
            else:
                self.uiRemoteMainServerHostLineEdit.setText(local_server_settings["host"])
                self.uiRemoteMainServerUserLineEdit.setText(local_server_settings["username"])
                self.uiRemoteMainServerPasswordLineEdit.setText(local_server_settings["password"])
            self.uiRemoteMainServerPortSpinBox.setValue(local_server_settings["port"])
        elif self.page(page_id) == self.uiSummaryWizardPage:
            self.uiSummaryTreeWidget.clear()
            if self.uiLocalControllerRadioButton.isChecked():
                local_server_settings = LocalServer.instance().localServerSettings()
                self._addSummaryEntry("Type:", "Local")
                self._addSummaryEntry("Path:", local_server_settings["path"])
                self._addSummaryEntry("Host:", local_server_settings["host"])
                self._addSummaryEntry("Port:", str(local_server_settings["port"]))
            elif self.uiRemoteControllerRadioButton.isChecked():
                local_server_settings = LocalServer.instance().localServerSettings()
                self._addSummaryEntry("Type:", "Remote")
                self._addSummaryEntry("Host:", local_server_settings["host"])
                self._addSummaryEntry("Port:", str(local_server_settings["port"]))
                self._addSummaryEntry("User:", local_server_settings["username"])

    def _saveSettingsCallback(self, result, error=False, **kwargs):
        if error:
            if "message" in result:
                QtWidgets.QMessageBox.critical(self, "Save settings", "Error while saving settings: {}".format(result["message"]))
            return

    def _addSummaryEntry(self, name, value):

        item = QtWidgets.QTreeWidgetItem(self.uiSummaryTreeWidget, [name, value])
        item.setText(0, name)
        font = item.font(0)
        font.setBold(True)
        item.setFont(0, font)

    def validateCurrentPage(self):
        """
        Validates the settings.
        """

        Controller.instance().setDisplayError(True)
        if self.currentPage() == self.uiLocalControllerWizardPage:

            local_controller_settings = LocalServer.instance().localServerSettings()
            local_controller_settings["auto_start"] = True
            local_controller_settings["remote"] = False
            local_controller_settings["path"] = self.uiLocalServerPathLineEdit.text().strip()
            local_controller_settings["host"] = self.uiLocalServerHostComboBox.itemData(self.uiLocalServerHostComboBox.currentIndex())
            local_controller_settings["port"] = self.uiLocalServerPortSpinBox.value()

            if not os.path.isfile(local_controller_settings["path"]):
                QtWidgets.QMessageBox.critical(self, "Local server", "Could not find local server {}".format(local_controller_settings["path"]))
                return False
            if not os.access(local_controller_settings["path"], os.X_OK):
                QtWidgets.QMessageBox.critical(self, "Local server", "{} is not an executable".format(local_controller_settings["path"]))
                return False

            LocalServer.instance().updateLocalServerSettings(local_controller_settings)

            # start and connect to the controller if required
            if not LocalServer.instance().localServerAutoStartIfRequired():
                return False

        elif self.currentPage() == self.uiRemoteControllerWizardPage:

            remote_controller_settings = Controller.instance().settings()
            remote_controller_settings["auto_start"] = False
            remote_controller_settings["remote"] = True
            remote_controller_settings["host"] = self.uiRemoteMainServerHostLineEdit.text()
            remote_controller_settings["port"] = self.uiRemoteMainServerPortSpinBox.value()
            remote_controller_settings["protocol"] = "http"
            remote_controller_settings["username"] = self.uiRemoteMainServerUserLineEdit.text()
            remote_controller_settings["password"] = self.uiRemoteMainServerPasswordLineEdit.text()
            Controller.instance().setSettings(remote_controller_settings)
            Controller.instance().connect()
            return Controller.instance().connected()

        return True

    def done(self, result):
        """
        This dialog is closed.

        :param result: ignored
        """

        Controller.instance().setDisplayError(True)
        settings = self.parentWidget().settings()
        if result:
            settings["hide_setup_wizard"] = True
        else:
            local_server_settings = LocalServer.instance().localServerSettings()
            LocalServer.instance().updateLocalServerSettings(local_server_settings)
            settings["hide_setup_wizard"] = not self.uiShowCheckBox.isChecked()
        self.parentWidget().setSettings(settings)
        super().done(result)

    def nextId(self):
        """
        Wizard rules!
        """

        current_id = self.currentId()
        if self.page(current_id) == self.uiLocalControllerWizardPage and self.uiLocalControllerRadioButton.isChecked():
            return self._pageId(self.uiSummaryWizardPage)
        if self.page(current_id) == self.uiControllerWizardPage and self.uiRemoteControllerRadioButton.isChecked():
            return self._pageId(self.uiRemoteControllerWizardPage)
        return QtWidgets.QWizard.nextId(self)

    def _pageId(self, page):
        """
        Return id of the page
        """

        for id in self.pageIds():
            if self.page(id) == page:
                return id
        raise KeyError



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .qt import QtCore
from .controller import Controller
from .local_config import LocalConfig
from .settings import GENERAL_SETTINGS
from .http_client_error import HttpClientError


import logging
log = logging.getLogger(__name__)


class ApplianceManager(QtCore.QObject):
    """
    Manager for appliances.
    """

    appliances_changed_signal = QtCore.Signal()

    def __init__(self):

        super().__init__()
        self._appliances = []
        self._controller = Controller.instance()
        self._controller.connected_signal.connect(self.refresh)
        self._controller.disconnected_signal.connect(self._controllerDisconnectedSlot)

    def refresh(self, update=False):
        """
        Gets the appliances from the controller.
        """

        if self._controller.connected():
            settings = LocalConfig.instance().loadSectionSettings("MainWindow", GENERAL_SETTINGS)
            symbol_theme = settings["symbol_theme"]
            if update is True:
                try:
                    self._controller.get(
                        "/appliances?update=yes&symbol_theme={}".format(symbol_theme),
                        self._listAppliancesCallback,
                        progress_text="Downloading appliances from online registry...",
                        wait=True,
                        timeout=300
                    )
                except HttpClientError as e:
                    log.error(f"Error while getting appliances list: {e}")
                    return
            else:
                self._controller.get("/appliances?symbol_theme={}".format(symbol_theme), self._listAppliancesCallback)

    def _controllerDisconnectedSlot(self):
        """
        Called when the controller has been disconnected.
        """

        self._appliances = []
        self.appliances_changed_signal.emit()

    def appliances(self):
        """
        Returns the appliances.

        :returns: array of appliances
        """

        return self._appliances

    def _listAppliancesCallback(self, result, error=False, **kwargs):
        """
        Callback to get the appliances.
        """

        if error is True:
            log.error("Error while getting appliances list: {}".format(result.get("message", "unknown")))
            return
        self._appliances = result
        self.appliances_changed_signal.emit()

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of ApplianceManager.
        :returns: instance of ApplianceManager
        """

        if not hasattr(ApplianceManager, '_instance') or ApplianceManager._instance is None:
            ApplianceManager._instance = ApplianceManager()
        return ApplianceManager._instance



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import xml.etree.ElementTree as ET

from gns3.items.image_item import ImageItem
from gns3.qt import QtGui, QtCore


@pytest.fixture
def image(project, controller):
    path = "resources/images/gns3_icon_128x128.png"
    return ImageItem(image_path=path, project=project)


def test_toSvg(image):
    svg = ET.fromstring(image.toSvg())
    assert float(svg.get("width")) ==  128.0
    assert float(svg.get("height")) ==  128.0


def test_fromSvg(image, project):
    image2 = ImageItem(svg=image.toSvg(), project=project)
    assert image2.toSvg() == image.toSvg()


def test_json(image):
    image._hash_svg = None
    assert "svg" in  image.__json__()
    # If we call the function twice and the svg didn't change we don't send the modification
    assert "svg" not in  image.__json__()





# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import psutil
import platform
import os
import stat
import struct

from gns3.qt import QtWidgets
from gns3.ui.doctor_dialog_ui import Ui_DoctorDialog
from gns3.local_server import LocalServer
from gns3.local_config import LocalConfig
from gns3 import version
from gns3.modules.vmware import VMware

import logging
log = logging.getLogger(__name__)


class DoctorDialog(QtWidgets.QDialog, Ui_DoctorDialog):
    """
    This dialog allow user to detect error in his GNS3 installation.

    If you want to add a test add a method starting by check. The
    check return a tuple result and a message in case of failure.
    """

    def __init__(self, parent, console=False):

        super().__init__(parent)
        self._console = console
        self.setupUi(self)
        self.uiOkButton.clicked.connect(self._okButtonClickedSlot)
        for method in sorted(dir(self)):
            if method.startswith('check'):
                try:
                    self.write(getattr(self, method).__doc__ + "...")
                    (res, msg) = getattr(self, method)()
                    if res == 0:
                        self.write('<span style="color: green"><strong>OK</strong></span>')
                    elif res == 1:
                        self.write('<span style="color: orange"><strong>WARNING</strong> {}</span>'.format(msg))
                    elif res == 2:
                        self.write('<span style="color: red"><strong>ERROR</strong> {}</span>'.format(msg))
                except Exception as e:
                    log.error("GNS3 doctor exception detected: {}".format(e), exc_info=1)
                    self.write('<span style="color: red"><strong>FAIL</strong> The doctor failed during this test with error: {} Please check on the forum.</span>'.format(str(e)))
                self.write("<br/>")

    def write(self, text):
        """
        Add text to the text windows
        """
        if self._console:
            print(text)
        self.uiDoctorResultTextEdit.setHtml(self.uiDoctorResultTextEdit.toHtml() + text)

    def _okButtonClickedSlot(self):
        self.accept()

    def checkLocalServerEnabled(self):
        """Checking if the local server is enabled"""
        if LocalServer.instance().shouldLocalServerAutoStart() is False:
            return (2, "The local server is disabled. Go to Preferences -> Server -> Local Server and enable the local server.")
        return (0, None)

    def checkDevVersionOfGNS3(self):
        """Checking for stable GNS3 version"""
        if version.__version_info__[3] != 0:
            return (1, "You are using a unstable version of GNS3.")
        return (0, None)

    def checkExperimentalFeaturesEnabled(self):
        """Checking if experimental features are not enabled"""
        if LocalConfig.instance().experimental():
            return (1, "Experimental features are enabled. Turn them off by going to Preferences -> General -> Miscellaneous.")
        return (0, None)

    def checkFreeRam(self):
        """Checking for amount of free virtual memory"""

        if int(psutil.virtual_memory().available / (1024 * 1024)) < 600:
            return (2, "You have less than 600MB of available virtual memory, this could prevent nodes to start")
        return (0, None)

    def checkVmrun(self):
        """Checking if vmrun is installed"""
        vmrun = VMware.instance().findVmrun()
        if len(vmrun) == 0:
            return (1, "The vmrun executable could not be found, VMware VMs cannot be used")
        return (0, None)

    def check64Bit(self):
        """Check if processor is 64 bit"""
        if platform.architecture()[0] != "64bit":
            return (2, "The architecture {} is not supported.".format(platform.architecture()[0]))
        return (0, None)

    def checkUbridgePermission(self):
        """Check if ubridge has the correct permission"""
        if not sys.platform.startswith("win") and os.geteuid() == 0:
            # we are root, so we should have privileged access.
            return (0, None)

        path = LocalServer.instance().localServerSettings().get("ubridge_path")
        if path is None:
            return (0, None)
        if not os.path.exists(path):
            return (2, "Ubridge path {path} doesn't exists".format(path=path))

        if sys.platform.startswith("linux"):
            try:
                if "security.capability" not in os.listxattr(path) or not struct.unpack("<IIIII", os.getxattr(path, "security.capability"))[1] & 1 << 13:
                    return (2, "Ubridge requires CAP_NET_RAW. Run sudo setcap cap_net_admin,cap_net_raw=ep {path}".format(path=path))
            except (OSError, AttributeError) as e:
                # Due to a Python bug, os.listxattr could be missing: https://github.com/GNS3/gns3-gui/issues/2010
                return (1, "Could not determine if CAP_NET_RAW capability is set for uBridge: {}".format(e))

        if sys.platform.startswith("darwin"):
            if os.stat(path).st_uid != 0 or not os.stat(path).st_mode & stat.S_ISUID:
                return (2, "Ubridge should be setuid. Run sudo chown root:admin {path} and sudo chmod 4750 {path}".format(path=path))
        return (0, None)

    def checkDynamipsPermission(self):
        """Check if dynamips has the correct permission"""
        if not sys.platform.startswith("win") and os.geteuid() == 0:
            # we are root, so we should have privileged access.
            return (0, None)

        path = LocalServer.instance().localServerSettings().get("dynamips_path")
        if path is None:
            return (0, None)
        if not os.path.exists(path):
            return (2, "Dynamips path {path} doesn't exists".format(path=path))

        try:
            if sys.platform.startswith("linux") and "security.capability" in os.listxattr(path):
                caps = os.getxattr(path, "security.capability")
                # test the 2nd byte and check if the 13th bit (CAP_NET_RAW) is set
                if not struct.unpack("<IIIII", caps)[1] & 1 << 13:
                    return (2, "Dynamips requires CAP_NET_RAW. Run sudo setcap cap_net_raw,cap_net_admin+eip {path}".format(path=path))
        except AttributeError:
            # Due to a Python bug, os.listxattr could be missing: https://github.com/GNS3/gns3-gui/issues/2010
            return (1, "Could not determine if CAP_NET_RAW capability is set for Dynamips (Python bug)".format(path=path))
        return (0, None)

    def checkGNS3InstalledTwice(self):
        """Check if gns3 is not installed twice"""

        if not sys.platform.startswith("win"):
            return (0, None)

        try:
            if os.path.exists("/usr/local/bin/gns3server") and os.path.exists("/usr/bin/gns3server"):
                return (2, "GNS3 is installed twice please remove it from /usr/local/bin")
        except OSError:
            pass
        return (0, None)


if __name__ == '__main__':

    app = QtWidgets.QApplication(sys.argv)
    main = QtWidgets.QMainWindow()
    dialog = DoctorDialog(main, console=True)
    # dialog.show()
    #exit_code = app.exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/vmware/ui/vmware_vm_configuration_page.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_VMwareVMConfigPageWidget(object):
    def setupUi(self, VMwareVMConfigPageWidget):
        VMwareVMConfigPageWidget.setObjectName("VMwareVMConfigPageWidget")
        VMwareVMConfigPageWidget.resize(767, 662)
        self.verticalLayout = QtWidgets.QVBoxLayout(VMwareVMConfigPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiTabWidget = QtWidgets.QTabWidget(VMwareVMConfigPageWidget)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout = QtWidgets.QGridLayout(self.tab)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.tab)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.tab)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiDefaultNameFormatLabel = QtWidgets.QLabel(self.tab)
        self.uiDefaultNameFormatLabel.setObjectName("uiDefaultNameFormatLabel")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLabel, 1, 0, 1, 1)
        self.uiDefaultNameFormatLineEdit = QtWidgets.QLineEdit(self.tab)
        self.uiDefaultNameFormatLineEdit.setObjectName("uiDefaultNameFormatLineEdit")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLineEdit, 1, 1, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(self.tab)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout.addWidget(self.uiSymbolLabel, 2, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(self.tab)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(self.tab)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_7, 2, 1, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(self.tab)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout.addWidget(self.uiCategoryLabel, 3, 0, 1, 1)
        self.uiCategoryComboBox = QtWidgets.QComboBox(self.tab)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.gridLayout.addWidget(self.uiCategoryComboBox, 3, 1, 1, 1)
        self.uiConsoleTypeLabel = QtWidgets.QLabel(self.tab)
        self.uiConsoleTypeLabel.setObjectName("uiConsoleTypeLabel")
        self.gridLayout.addWidget(self.uiConsoleTypeLabel, 4, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiConsoleTypeComboBox = QtWidgets.QComboBox(self.tab)
        self.uiConsoleTypeComboBox.setObjectName("uiConsoleTypeComboBox")
        self.uiConsoleTypeComboBox.addItem("")
        self.uiConsoleTypeComboBox.addItem("")
        self.horizontalLayout.addWidget(self.uiConsoleTypeComboBox)
        self.uiConsoleAutoStartCheckBox = QtWidgets.QCheckBox(self.tab)
        self.uiConsoleAutoStartCheckBox.setObjectName("uiConsoleAutoStartCheckBox")
        self.horizontalLayout.addWidget(self.uiConsoleAutoStartCheckBox)
        self.gridLayout.addLayout(self.horizontalLayout, 4, 1, 1, 1)
        self.uiOnCloseLabel = QtWidgets.QLabel(self.tab)
        self.uiOnCloseLabel.setObjectName("uiOnCloseLabel")
        self.gridLayout.addWidget(self.uiOnCloseLabel, 5, 0, 1, 1)
        self.uiOnCloseComboBox = QtWidgets.QComboBox(self.tab)
        self.uiOnCloseComboBox.setObjectName("uiOnCloseComboBox")
        self.gridLayout.addWidget(self.uiOnCloseComboBox, 5, 1, 1, 1)
        self.uiHeadlessModeCheckBox = QtWidgets.QCheckBox(self.tab)
        self.uiHeadlessModeCheckBox.setChecked(False)
        self.uiHeadlessModeCheckBox.setObjectName("uiHeadlessModeCheckBox")
        self.gridLayout.addWidget(self.uiHeadlessModeCheckBox, 6, 0, 1, 2)
        self.uiBaseVMCheckBox = QtWidgets.QCheckBox(self.tab)
        self.uiBaseVMCheckBox.setEnabled(True)
        self.uiBaseVMCheckBox.setObjectName("uiBaseVMCheckBox")
        self.gridLayout.addWidget(self.uiBaseVMCheckBox, 7, 0, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(20, 138, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 8, 0, 1, 1)
        self.uiTabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiAdaptersLabel = QtWidgets.QLabel(self.tab_2)
        self.uiAdaptersLabel.setObjectName("uiAdaptersLabel")
        self.gridLayout_2.addWidget(self.uiAdaptersLabel, 0, 0, 1, 1)
        self.uiFirstPortNameLabel = QtWidgets.QLabel(self.tab_2)
        self.uiFirstPortNameLabel.setObjectName("uiFirstPortNameLabel")
        self.gridLayout_2.addWidget(self.uiFirstPortNameLabel, 1, 0, 1, 1)
        self.uiPortNameFormatLabel = QtWidgets.QLabel(self.tab_2)
        self.uiPortNameFormatLabel.setObjectName("uiPortNameFormatLabel")
        self.gridLayout_2.addWidget(self.uiPortNameFormatLabel, 2, 0, 1, 1)
        self.uiPortSegmentSizeLabel = QtWidgets.QLabel(self.tab_2)
        self.uiPortSegmentSizeLabel.setObjectName("uiPortSegmentSizeLabel")
        self.gridLayout_2.addWidget(self.uiPortSegmentSizeLabel, 3, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.tab_2)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 4, 0, 1, 1)
        self.uiCustomAdaptersLabel = QtWidgets.QLabel(self.tab_2)
        self.uiCustomAdaptersLabel.setObjectName("uiCustomAdaptersLabel")
        self.gridLayout_2.addWidget(self.uiCustomAdaptersLabel, 5, 0, 1, 2)
        self.uiCustomAdaptersConfigurationPushButton = QtWidgets.QPushButton(self.tab_2)
        self.uiCustomAdaptersConfigurationPushButton.setObjectName("uiCustomAdaptersConfigurationPushButton")
        self.gridLayout_2.addWidget(self.uiCustomAdaptersConfigurationPushButton, 5, 2, 1, 1)
        self.uiUseAnyAdapterCheckBox = QtWidgets.QCheckBox(self.tab_2)
        self.uiUseAnyAdapterCheckBox.setObjectName("uiUseAnyAdapterCheckBox")
        self.gridLayout_2.addWidget(self.uiUseAnyAdapterCheckBox, 6, 0, 1, 3)
        spacerItem1 = QtWidgets.QSpacerItem(248, 178, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem1, 7, 1, 1, 2)
        self.uiAdapterTypesComboBox = QtWidgets.QComboBox(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiAdapterTypesComboBox.sizePolicy().hasHeightForWidth())
        self.uiAdapterTypesComboBox.setSizePolicy(sizePolicy)
        self.uiAdapterTypesComboBox.setObjectName("uiAdapterTypesComboBox")
        self.gridLayout_2.addWidget(self.uiAdapterTypesComboBox, 4, 2, 1, 1)
        self.uiPortSegmentSizeSpinBox = QtWidgets.QSpinBox(self.tab_2)
        self.uiPortSegmentSizeSpinBox.setMaximum(128)
        self.uiPortSegmentSizeSpinBox.setSingleStep(4)
        self.uiPortSegmentSizeSpinBox.setObjectName("uiPortSegmentSizeSpinBox")
        self.gridLayout_2.addWidget(self.uiPortSegmentSizeSpinBox, 3, 2, 1, 1)
        self.uiPortNameFormatLineEdit = QtWidgets.QLineEdit(self.tab_2)
        self.uiPortNameFormatLineEdit.setText("")
        self.uiPortNameFormatLineEdit.setObjectName("uiPortNameFormatLineEdit")
        self.gridLayout_2.addWidget(self.uiPortNameFormatLineEdit, 2, 2, 1, 1)
        self.uiFirstPortNameLineEdit = QtWidgets.QLineEdit(self.tab_2)
        self.uiFirstPortNameLineEdit.setObjectName("uiFirstPortNameLineEdit")
        self.gridLayout_2.addWidget(self.uiFirstPortNameLineEdit, 1, 2, 1, 1)
        self.uiAdaptersSpinBox = QtWidgets.QSpinBox(self.tab_2)
        self.uiAdaptersSpinBox.setMinimum(0)
        self.uiAdaptersSpinBox.setMaximum(10)
        self.uiAdaptersSpinBox.setObjectName("uiAdaptersSpinBox")
        self.gridLayout_2.addWidget(self.uiAdaptersSpinBox, 0, 2, 1, 1)
        self.uiTabWidget.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab_3)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiUsageTextEdit = QtWidgets.QPlainTextEdit(self.tab_3)
        self.uiUsageTextEdit.setObjectName("uiUsageTextEdit")
        self.verticalLayout_2.addWidget(self.uiUsageTextEdit)
        self.uiTabWidget.addTab(self.tab_3, "")
        self.verticalLayout.addWidget(self.uiTabWidget)

        self.retranslateUi(VMwareVMConfigPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(VMwareVMConfigPageWidget)

    def retranslateUi(self, VMwareVMConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        VMwareVMConfigPageWidget.setWindowTitle(_translate("VMwareVMConfigPageWidget", "VMware VM template configuration"))
        self.uiNameLabel.setText(_translate("VMwareVMConfigPageWidget", "Name:"))
        self.uiDefaultNameFormatLabel.setText(_translate("VMwareVMConfigPageWidget", "Default name format:"))
        self.uiSymbolLabel.setText(_translate("VMwareVMConfigPageWidget", "Symbol:"))
        self.uiSymbolToolButton.setText(_translate("VMwareVMConfigPageWidget", "&Browse..."))
        self.uiCategoryLabel.setText(_translate("VMwareVMConfigPageWidget", "Category:"))
        self.uiConsoleTypeLabel.setText(_translate("VMwareVMConfigPageWidget", "Console type:"))
        self.uiConsoleTypeComboBox.setItemText(0, _translate("VMwareVMConfigPageWidget", "telnet"))
        self.uiConsoleTypeComboBox.setItemText(1, _translate("VMwareVMConfigPageWidget", "none"))
        self.uiConsoleAutoStartCheckBox.setText(_translate("VMwareVMConfigPageWidget", "Auto start console"))
        self.uiOnCloseLabel.setText(_translate("VMwareVMConfigPageWidget", "On close:"))
        self.uiHeadlessModeCheckBox.setText(_translate("VMwareVMConfigPageWidget", "Start VM in headless mode"))
        self.uiBaseVMCheckBox.setText(_translate("VMwareVMConfigPageWidget", "Use as a linked base VM (experimental)"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.tab), _translate("VMwareVMConfigPageWidget", "General settings"))
        self.uiAdaptersLabel.setText(_translate("VMwareVMConfigPageWidget", "Adapters:"))
        self.uiFirstPortNameLabel.setText(_translate("VMwareVMConfigPageWidget", "First port name:"))
        self.uiPortNameFormatLabel.setToolTip(_translate("VMwareVMConfigPageWidget", "<html><head/><body><p>{0} - the port number, from 0 to the number of adapters-1.</p><p>{1} - the segment number, from 0 to the number of segments-1.</p><p>{port0} - named alias for {0}.</p><p>{port1} - the port number, from 1 to the number of adapters.</p><p>{segment0} - named alias for {1}.</p><p>{segment1} - the segment number, from 1 to the number of segments.</p></body></html>"))
        self.uiPortNameFormatLabel.setText(_translate("VMwareVMConfigPageWidget", "Name format:"))
        self.uiPortSegmentSizeLabel.setText(_translate("VMwareVMConfigPageWidget", "Segment size:"))
        self.label.setText(_translate("VMwareVMConfigPageWidget", "Type:"))
        self.uiCustomAdaptersLabel.setText(_translate("VMwareVMConfigPageWidget", "Custom adapters:"))
        self.uiCustomAdaptersConfigurationPushButton.setText(_translate("VMwareVMConfigPageWidget", "&Configure custom adapters"))
        self.uiUseAnyAdapterCheckBox.setToolTip(_translate("VMwareVMConfigPageWidget", "<html><head/><body><p>This option will allow GNS3 to replace all adapters of the VM. This allow you to use any interface of the VM in GNS3.</p></body></html>"))
        self.uiUseAnyAdapterCheckBox.setText(_translate("VMwareVMConfigPageWidget", "Allow GNS3 to override non custom VMware adapter"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.tab_2), _translate("VMwareVMConfigPageWidget", "Network"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.tab_3), _translate("VMwareVMConfigPageWidget", "Usage"))




#!/usr/bin/env python
#
# Copyright (C) 2023 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This script is intended to be built as a small executable for macOS to set the correct permissions on uBridge

import os
import shutil
import sys


def authorize_ubridge():

    path = shutil.which("ubridge", path=os.path.dirname(sys.executable))
    if path is None:
        raise SystemExit("Could not find ubridge executable at {}".format(path))
    try:
        shutil.chown(path, "root", "admin")
        os.chmod(path, 0o4750)
    except OSError as e:
        raise SystemExit("Could not authorize {}: {}".format(path, str(e)))


if __name__ == '__main__':
    authorize_ubridge()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gns3.utils import parse_version

from ..qt import QtGui, QtWidgets, QtCore, qslot, qpartial
from ..topology import Topology
from ..ui.edit_project_dialog_ui import Ui_EditProjectDialog


class EditProjectDialog(QtWidgets.QDialog, Ui_EditProjectDialog):
    """
    Edit current project settings
    """

    def __init__(self, parent):

        super().__init__(parent)
        self.setupUi(self)
        self._project = Topology.instance().project()
        self.uiProjectNameLineEdit.setText(self._project.name())
        self.uiProjectAutoOpenCheckBox.setChecked(self._project.autoOpen())
        self.uiProjectAutoCloseCheckBox.setChecked(not self._project.autoClose())
        self.uiProjectAutoStartCheckBox.setChecked(self._project.autoStart())
        self.uiSceneWidthSpinBox.setValue(self._project.sceneWidth())
        self.uiSceneHeightSpinBox.setValue(self._project.sceneHeight())
        self.uiNodeGridSizeSpinBox.setValue(self._project.nodeGridSize())
        self.uiDrawingGridSizeSpinBox.setValue(self._project.drawingGridSize())

        self.uiGlobalVariablesGrid.setAlignment(QtCore.Qt.AlignTop)

        self.uiNewVarButton = QtWidgets.QPushButton('Add new variable', self)
        self.uiNewVarButton.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        self.uiNewVarButton.clicked.connect(self.onAddNewVariable)
        self.uiGlobalVariablesGrid.addWidget(self.uiNewVarButton, 0, 3, QtCore.Qt.AlignRight)

        self._readme_filename = "README.txt"
        self.uiTabWidget.currentChanged.connect(self._previewMarkdownSlot)
        self._loadReadme()
        self._variables = self._project.variables()
        if not self._variables:
            self._variables = [{"name": "", "value": ""}]
        self.updateGlobalVariables()

    def _loadReadme(self):

        self._project.get("/files/{}".format(self._readme_filename), self._loadedReadme, raw=True)

    def _loadedReadme(self, result, error=False, context={}, **kwargs):

        if not error:
            content = result.decode("utf-8", errors="replace")
            self.uiReadmeTextEdit.setPlainText(content)

    def _previewMarkdownSlot(self, index):

        # index 1 is preview tab
        if index == 1:

            # QTextDocument before Qt version 5.14 doesn't support Markdown
            if parse_version(QtCore.QT_VERSION_STR) < parse_version("5.14.0") or parse_version(QtCore.PYQT_VERSION_STR) < parse_version("5.14.0"):
                QtWidgets.QMessageBox.critical(self, "Markdown preview", "Markdown preview is only support with Qt version 5.14.0 or above")
                return

            # show Markdown preview
            document = QtGui.QTextDocument()
            self.uiReadmePreview.setDocument(document)
            document.setMarkdown(self.uiReadmeTextEdit.toPlainText())

    def updateGlobalVariables(self):
        while True:
            item = self.uiGlobalVariablesGrid.takeAt(1)
            if item is None:
                break
            elif item.widget():
                item.widget().deleteLater()

        for i, variable in enumerate(self._variables, start=1):
            nameLabel = QtWidgets.QLabel()
            nameLabel.setText("Name:")
            self.uiGlobalVariablesGrid.addWidget(nameLabel, i, 0)

            nameEdit = QtWidgets.QLineEdit()
            nameEdit.setText(variable.get("name", ""))
            nameEdit.textChanged.connect(qpartial(self.onNameChange, variable))
            self.uiGlobalVariablesGrid.addWidget(nameEdit, i, 1)

            valueLabel = QtWidgets.QLabel()
            valueLabel.setText("Value:")
            self.uiGlobalVariablesGrid.addWidget(valueLabel, i, 2)

            valueEdit = QtWidgets.QLineEdit()
            valueEdit.setText(variable.get("value", ""))
            valueEdit.textChanged.connect(qpartial(self.onValueChange, variable))
            self.uiGlobalVariablesGrid.addWidget(valueEdit, i, 3)

    @qslot
    def onAddNewVariable(self, event):
        self._variables += [{"name": "", "value": ""}]
        self.updateGlobalVariables()

    def onNameChange(self, variable, text):
        variable["name"] = text

    def onValueChange(self, variable, text):
        variable["value"] = text

    def _cleanVariables(self):
        return [v for v in self._variables if v.get("name").strip() != ""]

    def done(self, result):
        """
        Called when the dialog is closed.

        :param result: boolean (accepted or rejected)
        """

        if result:
            node_grid_size = self.uiNodeGridSizeSpinBox.value()
            drawing_grid_size = self.uiDrawingGridSizeSpinBox.value()
            if node_grid_size % drawing_grid_size != 0:
                QtWidgets.QMessageBox.critical(self, "Grid sizes", "Invalid grid sizes which will create overlapping lines")
            else:
                self._project.setNodeGridSize(node_grid_size)
                self._project.setDrawingGridSize(drawing_grid_size)
                self._project.setName(self.uiProjectNameLineEdit.text())
                self._project.setAutoOpen(self.uiProjectAutoOpenCheckBox.isChecked())
                self._project.setAutoClose(not self.uiProjectAutoCloseCheckBox.isChecked())
                self._project.setAutoStart(self.uiProjectAutoStartCheckBox.isChecked())
                self._project.setSceneHeight(self.uiSceneHeightSpinBox.value())
                self._project.setSceneWidth(self.uiSceneWidthSpinBox.value())
                self._project.setVariables(self._cleanVariables())
                self._project.update()
                content = self.uiReadmeTextEdit.toPlainText()
                if content:
                    self._project.post("/files/{}".format(self._readme_filename), self._saveReadmeCallback, body=content)
        super().done(result)

    def _saveReadmeCallback(self, result, error=False, **kwargs):

        if error:
            QtWidgets.QMessageBox.critical(self, "Edit project", "Could not created readme file")



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import sys
import shutil
import tarfile
import os


from gns3.update_manager import UpdateManager
from gns3 import version


@pytest.fixture
def frozen():
    sys.frozen = True
    yield
    delattr(sys, 'frozen')


@pytest.fixture
def devVersion():
    old_version_info = version.__version_info__
    old_version = version.__version__
    version.__version_info__ = (1, 4, 0, -99)
    version.__version__ = '1.4.0'
    yield
    version.__version_info__ = old_version
    version.__version__ = old_version


@pytest.fixture
def stableVersion():
    old_version_info = version.__version_info__
    old_version = version.__version__
    version.__version_info__ = (1, 4, 0, 0)
    version.__version__ = '1.4.0'
    yield
    version.__version_info__ = old_version_info
    version.__version__ = old_version


@pytest.fixture
def update(tmpdir):
    update = UpdateManager()
    update._update_directory = str(tmpdir / 'updates')
    update._package_directory = str(tmpdir / 'site-packages')
    return update


@pytest.fixture
def gui_tar(tmpdir):
    tgz = str(tmpdir / 'updates' / 'gns3-gui.tar.gz')
    os.makedirs(str(tmpdir / 'updates'), exist_ok=True)
    os.makedirs(str(tmpdir / 'gns3-gui-master' / 'gns3'))
    open(str(tmpdir / 'gns3-gui-master' / 'gns3' / 'test.py'), 'w+').close()
    with tarfile.open(tgz, 'w:gz') as tar:
        tar.add(str(tmpdir / 'gns3-gui-master'), arcname='gns3-gui-master')
    shutil.rmtree(str(tmpdir / 'gns3-gui-master' / 'gns3'))
    return tgz


def test_isDevVersion(devVersion, update):
    assert update.isDevVersion() is True


def test_isDevVersion_stable(stableVersion, update):
    assert update.isDevVersion() is False


def test_installDownloadedUpdates(frozen, update, gui_tar, tmpdir):
    update.installDownloadedUpdates()
    assert os.path.exists(str(tmpdir / 'site-packages' / 'gns3' / 'test.py'))


def test_getLastVersionFromPyPiReply_Stable(update, stableVersion):
    body = {
        'releases': {
            '1.4.4rc1': [{'url': 'http://example.com'}],
            '1.4.3': [{'url': 'http://example.com'}],
            '1.4.1': [{'url': 'http://example.com'}],
            '1.4.2': [{'url': 'http://example.com'}]
        }
    }
    assert update._getLastMinorVersionFromPyPiReply(body) == '1.4.3'


def test_getLastVersionFromPyPiReply_Dev(update, devVersion):
    body = {
        'releases': {
            '1.4.2rc1': [{'url': 'http://example.com'}],
            '1.4.1': [{'url': 'http://example.com'}]
        }
    }
    assert update._getLastMinorVersionFromPyPiReply(body) == '1.4.2rc1'


def test_getLastVersionFromPyPiReplyNewMajor(update, stableVersion):
    body = {
        'releases': {
            '1.5': [{'url': 'http://example.com'}],
            '1.4.1': [{'url': 'http://example.com'}]
        }
    }
    assert update._getLastMinorVersionFromPyPiReply(body) == '1.4.1'



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import copy
import pathlib

from gns3.qt import QtWidgets
from gns3.controller import Controller
from gns3.utils.file_copy_worker import FileCopyWorker
from gns3.utils.progress_dialog import ProgressDialog
from gns3.http_client_error import HttpClientError, HttpClientCancelledRequestError
from gns3.registry.image import Image


class ImageManager:

    def __init__(self):
        # Remember if we already ask the user about this image for this server
        self._asked_for_this_image = {}

    def _getUniqueDestinationPath(self, source_image, node_type, path):
        """
        Get a unique destination path (with counter).
        """

        if not os.path.exists(path):
            return path
        path, extension = os.path.splitext(path)
        counter = 1
        new_path = "{}-{}{}".format(path, counter, extension)
        while os.path.exists(new_path):
            destination_image = Image(node_type, new_path, filename=os.path.basename(new_path))
            try:
                if source_image.md5sum == destination_image.md5sum:
                    # the source and destination images are identical
                    return new_path
            except OSError:
                continue
            counter += 1
            new_path = "{}-{}{}".format(path, counter, extension)
        return new_path

    def askCopyUploadImage(self, parent, source_path, server, node_type):
        """
        Ask user for copying the image to the default directory or upload
        it to remote server.

        :param parent: Parent window
        :param path: File path on computer
        :param server: The server where the images should be located
        :param node_type: Remote upload endpoint
        :returns path: Final path
        """

        if (server and server != "local") or Controller.instance().isRemote():
            return self._uploadImageToRemoteServer(source_path, node_type, parent)
        else:
            destination_directory = self.getDirectoryForType(node_type)
            destination_path = os.path.join(destination_directory, os.path.basename(source_path))
            source_filename = os.path.basename(source_path)
            destination_filename = os.path.basename(destination_path)
            if os.path.normpath(os.path.dirname(source_path)) != destination_directory:
                # the image is not in the default images directory
                if source_filename == destination_filename:
                    # the filename already exists in the default images directory
                    source_image = Image(node_type, source_path, filename=source_filename)
                    destination_image = Image(node_type, destination_path, filename=destination_filename)
                    try:
                        if source_image.md5sum == destination_image.md5sum:
                            # the source and destination images are identical
                            return source_path
                    except OSError as e:
                        QtWidgets.QMessageBox.critical(parent, 'Image', 'Cannot compare image file {} with {}: {}.'.format(source_path, destination_path, str(e)))
                        return source_path
                    # find a new unique path to avoid overwriting existing destination file
                    destination_path = self._getUniqueDestinationPath(source_image, node_type, destination_path)

                reply = QtWidgets.QMessageBox.question(parent,
                                                       'Image',
                                                       'Would you like to copy {} to the default images directory'.format(source_filename),
                                                       QtWidgets.QMessageBox.Yes,
                                                       QtWidgets.QMessageBox.No)
                if reply == QtWidgets.QMessageBox.Yes:
                    try:
                        os.makedirs(destination_directory, exist_ok=True)
                    except OSError as e:
                        QtWidgets.QMessageBox.critical(parent, 'Image', 'Could not create destination directory {}: {}'.format(destination_directory, str(e)))
                        return source_path

                    worker = FileCopyWorker(source_path, destination_path)
                    progress_dialog = ProgressDialog(worker, 'Image', 'Copying {}'.format(source_filename), 'Cancel', busy=True, parent=parent)
                    progress_dialog.show()
                    progress_dialog.exec_()
                    errors = progress_dialog.errors()
                    if errors:
                        QtWidgets.QMessageBox.critical(parent, 'Image', '{}'.format(''.join(errors)))
                        return source_path
                    else:
                        source_path = destination_path
            return source_path

    def _uploadImageToRemoteServer(self, path, node_type, parent):
        """
        Upload image to remote server

        :param path: File path on computer
        :param node_type: Image node_type
        :returns path: Final path
        """

        if node_type == 'QEMU':
            image_type = 'qemu'
        elif node_type == 'IOU':
            image_type = 'iou'
        elif node_type == 'DYNAMIPS':
            image_type = 'ios'
        else:
            raise Exception('Invalid node type')

        filename = self._getRelativeImagePath(path, node_type).replace("\\", "/")

        try:
            Controller.instance().post(
                f"/images/upload/{filename}",
                callback=None,
                params={"image_type": image_type},
                body=pathlib.Path(path),
                progress_text="Uploading {}".format(filename),
                timeout=None,
                wait=True
            )
        except HttpClientCancelledRequestError:
            return
        except HttpClientError as e:
            QtWidgets.QMessageBox.critical(
                parent,
                "Image upload",
                f"Could not upload image {filename}: {e}"
            )

        return filename

    def _getRelativeImagePath(self, path, node_type):
        """
        Get a path relative to images directory path
        or just filename if the path is not located inside
        image directory

        :param path: file path
        :param node_type: Type of vm
        :return: file path
        """

        if not path:
            return ""
        img_directory = self.getDirectoryForType(node_type)
        path = os.path.abspath(path)
        if os.path.commonprefix([img_directory, path]) == img_directory:
            return os.path.relpath(path, img_directory)
        return os.path.basename(path)

    def getDirectory(self):
        """
        Returns the images directory path.

        :returns: path to the default images directory
        """

        return copy.copy(Controller.instance().settings()['images_path'])

    def getDirectoryForType(self, node_type):
        """
        Return the path of local directory of the images
        of a specific node_type

        :param node_type: Type of vm
        """
        if node_type == 'DYNAMIPS':
            return os.path.join(self.getDirectory(), 'IOS')
        else:
            return os.path.join(self.getDirectory(), node_type.upper())

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of ImageManager.

        :returns: instance of ImageManager
        """

        if not hasattr(ImageManager, '_instance') or ImageManager._instance is None:
            ImageManager._instance = ImageManager()
        return ImageManager._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
QEMU VM implementation.
"""

import re

from gns3.node import Node
from .settings import QEMU_VM_SETTINGS


import logging
log = logging.getLogger(__name__)


class QemuVM(Node):
    """
    QEMU VM.

    :param module: parent module for this node
    :param server: GNS3 server instance
    """

    URL_PREFIX = "qemu"

    def __init__(self, module, server, project):
        super().__init__(module, server, project)

        self._linked_clone = True

        qemu_vm_settings = {"usage": "",
                            "hda_disk_image": "",
                            "hdb_disk_image": "",
                            "hdc_disk_image": "",
                            "hdd_disk_image": "",
                            "hda_disk_interface": QEMU_VM_SETTINGS["hda_disk_interface"],
                            "hdb_disk_interface": QEMU_VM_SETTINGS["hdb_disk_interface"],
                            "hdc_disk_interface": QEMU_VM_SETTINGS["hdc_disk_interface"],
                            "hdd_disk_interface": QEMU_VM_SETTINGS["hdd_disk_interface"],
                            "cdrom_image": "",
                            "bios_image": "",
                            "hda_disk_image_md5sum": "",
                            "hdb_disk_image_md5sum": "",
                            "hdc_disk_image_md5sum": "",
                            "hdd_disk_image_md5sum": "",
                            "cdrom_image_md5sum": "",
                            "bios_image_md5sum": "",
                            "boot_priority": QEMU_VM_SETTINGS["boot_priority"],
                            "options": "",
                            "ram": QEMU_VM_SETTINGS["ram"],
                            "cpus": QEMU_VM_SETTINGS["cpus"],
                            "console_type": QEMU_VM_SETTINGS["console_type"],
                            "console_auto_start": QEMU_VM_SETTINGS["console_auto_start"],
                            "aux_type": QEMU_VM_SETTINGS["aux_type"],
                            "adapters": QEMU_VM_SETTINGS["adapters"],
                            "custom_adapters": QEMU_VM_SETTINGS["custom_adapters"],
                            "adapter_type": QEMU_VM_SETTINGS["adapter_type"],
                            "mac_address": QEMU_VM_SETTINGS["mac_address"],
                            "replicate_network_connection_state": QEMU_VM_SETTINGS["replicate_network_connection_state"],
                            "tpm": QEMU_VM_SETTINGS["tpm"],
                            "uefi": QEMU_VM_SETTINGS["uefi"],
                            "create_config_disk": QEMU_VM_SETTINGS["create_config_disk"],
                            "platform": QEMU_VM_SETTINGS["platform"],
                            "qemu_path": "",
                            "on_close": QEMU_VM_SETTINGS["on_close"],
                            "cpu_throttling": QEMU_VM_SETTINGS["cpu_throttling"],
                            "process_priority": QEMU_VM_SETTINGS["process_priority"],
                            "initrd": "",
                            "kernel_image": "",
                            "initrd_md5sum": "",
                            "kernel_image_md5sum": "",
                            "kernel_command_line": "",
                            "port_name_format": "Ethernet{0}",
                            "port_segment_size": 0,
                            "first_port_name": ""}

        self.settings().update(qemu_vm_settings)

    def createDiskImage(self, disk_name, options, callback):
        """
        Create a disk image attached to the VM.

        :param callback: callback for the reply from the server
        """

        self.post(f"/qemu/disk_image/{disk_name}", callback, body=options)

    def resizeDiskImage(self, disk_name, size, callback):
        """
        Resize a disk image attached to the VM.

        :param callback: callback for the reply from the server
        """

        params = {"extend": size}
        self.put(f"/qemu/disk_image/{disk_name}", callback, body=params)

    def info(self):
        """
        Returns information about this QEMU VM instance.

        :returns: formatted string
        """

        info = """QEMU VM {name} is {state}
  Running on server {host} with port {port}
  Local ID is {id} and node ID is {node_id}
  Number of processors is {cpus} and amount of memory is {ram}MB
  Console is on port {console} and type is {console_type}
  Auxiliary console is on port {aux} and type is {aux_type}
""".format(name=self.name(),
           id=self.id(),
           node_id=self._node_id,
           state=self.state(),
           host=self.compute().name(),
           port=self.compute().port(),
           cpus=self._settings["cpus"],
           ram=self._settings["ram"],
           console=self._settings["console"],
           console_type=self._settings["console_type"],
           aux=self._settings["aux"],
           aux_type=self._settings["aux_type"])

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "     {port_name} is empty\n".format(port_name=port.name())
            else:
                port_info += "     {port_name} {port_description}\n".format(port_name=port.name(),
                                                                            port_description=port.description())
            if port.macAddress():
                port_info += "       MAC address is {mac_address}\n".format(mac_address=port.macAddress())

        usage = "\n" + self._settings.get("usage")
        return info + port_info + usage

    def configFiles(self):
        """
        Name of the configuration files
        """

        if self._settings.get("create_config_disk"):
            return ["config.zip"]
        return None

    def configTextFiles(self):
        """
        Name of the configuration files, which are plain text files

        :returns: List of configuration files, False if no files
        """

        return None

    def auxConsole(self):
        """
        Returns the console port for this Docker VM instance.

        :returns: port (integer)
        """

        return self._settings["aux"]

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from .pages.qemu_vm_configuration_page import QemuVMConfigurationPage
        return QemuVMConfigurationPage

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this node.

        :returns: symbol path (or resource).
        """

        return ":/symbols/qemu_guest.svg"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node categories
        """

        return [Node.end_devices]

    def __str__(self):

        return "QEMU VM"



#!/usr/bin/env python
#
# Copyright (C) 2023 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import shutil

import logging
log = logging.getLogger(__name__)


def macos_ubridge_setuid():

    # AuthorizationExecuteWithPrivileges() has been deprecated since macOS 10.7 but it still works
    # and much simpler than using SMJobBless() which requires a separate helper tool

    import ctypes
    import ctypes.util
    from ctypes import byref

    authorize_ubridge = shutil.which("authorize_ubridge", path=os.path.dirname(sys.executable))
    if authorize_ubridge is None:
        raise OSError("Could not find the authorize_ubridge executable")

    # https://developer.apple.com/documentation/security
    sec = ctypes.cdll.LoadLibrary(ctypes.util.find_library("Security"))

    try:
        sec.AuthorizationCreate
    except AttributeError:
        raise OSError("macOS security library does not support AuthorizationCreate")

    kAuthorizationFlagDefaults = 0
    auth = ctypes.c_void_p()
    r_auth = byref(auth)
    err = sec.AuthorizationCreate(None, None, kAuthorizationFlagDefaults, r_auth)
    if err:
        raise OSError("Could not create authorization: {}".format(err))

    exe = [authorize_ubridge]
    log.info("Executing '{}' with privileges".format(exe))
    args = (ctypes.c_char_p * len(exe))()
    for i, arg in enumerate(exe[1:]):
        args[i] = arg.encode('utf8')
    io = ctypes.c_void_p()
    err = sec.AuthorizationExecuteWithPrivileges(auth, exe[0].encode('utf8'), 0, args, byref(io))
    if err:
        raise OSError("Could not authorize uBridge: {}".format(err))
    else:
        log.info("Successfully authorized uBridge")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/show_readme_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ShowReadmeDialog(object):
    def setupUi(self, ShowReadmeDialog):
        ShowReadmeDialog.setObjectName("ShowReadmeDialog")
        ShowReadmeDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        ShowReadmeDialog.resize(682, 565)
        ShowReadmeDialog.setModal(False)
        self.verticalLayout = QtWidgets.QVBoxLayout(ShowReadmeDialog)
        self.verticalLayout.setContentsMargins(-1, -1, 12, -1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiTextBrowser = QtWidgets.QTextBrowser(ShowReadmeDialog)
        self.uiTextBrowser.setOpenExternalLinks(True)
        self.uiTextBrowser.setObjectName("uiTextBrowser")
        self.verticalLayout.addWidget(self.uiTextBrowser)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(20)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.uiRefreshButton = QtWidgets.QPushButton(ShowReadmeDialog)
        self.uiRefreshButton.setObjectName("uiRefreshButton")
        self.horizontalLayout.addWidget(self.uiRefreshButton)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(ShowReadmeDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiButtonBox.sizePolicy().hasHeightForWidth())
        self.uiButtonBox.setSizePolicy(sizePolicy)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.horizontalLayout.addWidget(self.uiButtonBox)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(ShowReadmeDialog)
        self.uiButtonBox.accepted.connect(ShowReadmeDialog.accept)
        self.uiButtonBox.rejected.connect(ShowReadmeDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(ShowReadmeDialog)

    def retranslateUi(self, ShowReadmeDialog):
        _translate = QtCore.QCoreApplication.translate
        ShowReadmeDialog.setWindowTitle(_translate("ShowReadmeDialog", "Readme"))
        self.uiRefreshButton.setText(_translate("ShowReadmeDialog", "Refresh"))
from . import resources_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/general_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_GeneralPreferencesPageWidget(object):
    def setupUi(self, GeneralPreferencesPageWidget):
        GeneralPreferencesPageWidget.setObjectName("GeneralPreferencesPageWidget")
        GeneralPreferencesPageWidget.resize(963, 1168)
        self.verticalLayout = QtWidgets.QVBoxLayout(GeneralPreferencesPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiMiscTabWidget = QtWidgets.QTabWidget(GeneralPreferencesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiMiscTabWidget.sizePolicy().hasHeightForWidth())
        self.uiMiscTabWidget.setSizePolicy(sizePolicy)
        self.uiMiscTabWidget.setObjectName("uiMiscTabWidget")
        self.uiGeneralTab = QtWidgets.QWidget()
        self.uiGeneralTab.setObjectName("uiGeneralTab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiGeneralTab)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.uiLocalPathsGroupBox = QtWidgets.QGroupBox(self.uiGeneralTab)
        self.uiLocalPathsGroupBox.setObjectName("uiLocalPathsGroupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiLocalPathsGroupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.uiProjectsPathLabel = QtWidgets.QLabel(self.uiLocalPathsGroupBox)
        self.uiProjectsPathLabel.setObjectName("uiProjectsPathLabel")
        self.gridLayout_4.addWidget(self.uiProjectsPathLabel, 0, 0, 1, 1)
        self.uiProjectsPathLineEdit = QtWidgets.QLineEdit(self.uiLocalPathsGroupBox)
        self.uiProjectsPathLineEdit.setObjectName("uiProjectsPathLineEdit")
        self.gridLayout_4.addWidget(self.uiProjectsPathLineEdit, 1, 0, 1, 1)
        self.uiProjectsPathToolButton = QtWidgets.QToolButton(self.uiLocalPathsGroupBox)
        self.uiProjectsPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiProjectsPathToolButton.setObjectName("uiProjectsPathToolButton")
        self.gridLayout_4.addWidget(self.uiProjectsPathToolButton, 1, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.uiLocalPathsGroupBox)
        self.label.setObjectName("label")
        self.gridLayout_4.addWidget(self.label, 2, 0, 1, 1)
        self.uiSymbolsPathLineEdit = QtWidgets.QLineEdit(self.uiLocalPathsGroupBox)
        self.uiSymbolsPathLineEdit.setObjectName("uiSymbolsPathLineEdit")
        self.gridLayout_4.addWidget(self.uiSymbolsPathLineEdit, 3, 0, 1, 1)
        self.uiSymbolsPathToolButton = QtWidgets.QToolButton(self.uiLocalPathsGroupBox)
        self.uiSymbolsPathToolButton.setObjectName("uiSymbolsPathToolButton")
        self.gridLayout_4.addWidget(self.uiSymbolsPathToolButton, 3, 1, 1, 1)
        self.uiConfigsPathLabel = QtWidgets.QLabel(self.uiLocalPathsGroupBox)
        self.uiConfigsPathLabel.setObjectName("uiConfigsPathLabel")
        self.gridLayout_4.addWidget(self.uiConfigsPathLabel, 4, 0, 1, 1)
        self.uiConfigsPathLineEdit = QtWidgets.QLineEdit(self.uiLocalPathsGroupBox)
        self.uiConfigsPathLineEdit.setObjectName("uiConfigsPathLineEdit")
        self.gridLayout_4.addWidget(self.uiConfigsPathLineEdit, 5, 0, 1, 1)
        self.uiConfigsPathToolButton = QtWidgets.QToolButton(self.uiLocalPathsGroupBox)
        self.uiConfigsPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiConfigsPathToolButton.setObjectName("uiConfigsPathToolButton")
        self.gridLayout_4.addWidget(self.uiConfigsPathToolButton, 5, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.uiLocalPathsGroupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 6, 0, 1, 2)
        self.uiAppliancesPathLineEdit = QtWidgets.QLineEdit(self.uiLocalPathsGroupBox)
        self.uiAppliancesPathLineEdit.setObjectName("uiAppliancesPathLineEdit")
        self.gridLayout_4.addWidget(self.uiAppliancesPathLineEdit, 7, 0, 1, 1)
        self.uiAppliancesPathToolButton = QtWidgets.QToolButton(self.uiLocalPathsGroupBox)
        self.uiAppliancesPathToolButton.setObjectName("uiAppliancesPathToolButton")
        self.gridLayout_4.addWidget(self.uiAppliancesPathToolButton, 7, 1, 1, 1)
        self.verticalLayout_4.addWidget(self.uiLocalPathsGroupBox)
        self.uiStyleGroupBox = QtWidgets.QGroupBox(self.uiGeneralTab)
        self.uiStyleGroupBox.setObjectName("uiStyleGroupBox")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout(self.uiStyleGroupBox)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.uiStyleComboBox = QtWidgets.QComboBox(self.uiStyleGroupBox)
        self.uiStyleComboBox.setObjectName("uiStyleComboBox")
        self.horizontalLayout_11.addWidget(self.uiStyleComboBox)
        self.verticalLayout_4.addWidget(self.uiStyleGroupBox)
        self.uiSymbolThemeGroupBox = QtWidgets.QGroupBox(self.uiGeneralTab)
        self.uiSymbolThemeGroupBox.setObjectName("uiSymbolThemeGroupBox")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout(self.uiSymbolThemeGroupBox)
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.uiSymbolThemeComboBox = QtWidgets.QComboBox(self.uiSymbolThemeGroupBox)
        self.uiSymbolThemeComboBox.setObjectName("uiSymbolThemeComboBox")
        self.horizontalLayout_15.addWidget(self.uiSymbolThemeComboBox)
        self.verticalLayout_4.addWidget(self.uiSymbolThemeGroupBox)
        self.uiConfigurationFileGroupBox = QtWidgets.QGroupBox(self.uiGeneralTab)
        self.uiConfigurationFileGroupBox.setObjectName("uiConfigurationFileGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiConfigurationFileGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiImportConfigurationFilePushButton = QtWidgets.QPushButton(self.uiConfigurationFileGroupBox)
        self.uiImportConfigurationFilePushButton.setObjectName("uiImportConfigurationFilePushButton")
        self.horizontalLayout.addWidget(self.uiImportConfigurationFilePushButton)
        self.uiExportConfigurationFilePushButton = QtWidgets.QPushButton(self.uiConfigurationFileGroupBox)
        self.uiExportConfigurationFilePushButton.setObjectName("uiExportConfigurationFilePushButton")
        self.horizontalLayout.addWidget(self.uiExportConfigurationFilePushButton)
        self.uiBrowseConfigurationPushButton = QtWidgets.QPushButton(self.uiConfigurationFileGroupBox)
        self.uiBrowseConfigurationPushButton.setObjectName("uiBrowseConfigurationPushButton")
        self.horizontalLayout.addWidget(self.uiBrowseConfigurationPushButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)
        self.uiConfigurationFileLabel = QtWidgets.QLabel(self.uiConfigurationFileGroupBox)
        self.uiConfigurationFileLabel.setObjectName("uiConfigurationFileLabel")
        self.gridLayout.addWidget(self.uiConfigurationFileLabel, 0, 0, 1, 1)
        self.verticalLayout_4.addWidget(self.uiConfigurationFileGroupBox)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem1)
        self.uiMiscTabWidget.addTab(self.uiGeneralTab, "")
        self.uiImagesTab = QtWidgets.QWidget()
        self.uiImagesTab.setObjectName("uiImagesTab")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.uiImagesTab)
        self.verticalLayout_7.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.uiLocalBinaryImagePathsGroupBox = QtWidgets.QGroupBox(self.uiImagesTab)
        self.uiLocalBinaryImagePathsGroupBox.setObjectName("uiLocalBinaryImagePathsGroupBox")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.uiLocalBinaryImagePathsGroupBox)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.uiImagesPathLabel = QtWidgets.QLabel(self.uiLocalBinaryImagePathsGroupBox)
        self.uiImagesPathLabel.setObjectName("uiImagesPathLabel")
        self.verticalLayout_10.addWidget(self.uiImagesPathLabel)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.uiImagesPathLineEdit = QtWidgets.QLineEdit(self.uiLocalBinaryImagePathsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiImagesPathLineEdit.sizePolicy().hasHeightForWidth())
        self.uiImagesPathLineEdit.setSizePolicy(sizePolicy)
        self.uiImagesPathLineEdit.setObjectName("uiImagesPathLineEdit")
        self.horizontalLayout_4.addWidget(self.uiImagesPathLineEdit)
        self.uiImagesPathToolButton = QtWidgets.QToolButton(self.uiLocalBinaryImagePathsGroupBox)
        self.uiImagesPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiImagesPathToolButton.setObjectName("uiImagesPathToolButton")
        self.horizontalLayout_4.addWidget(self.uiImagesPathToolButton)
        self.verticalLayout_10.addLayout(self.horizontalLayout_4)
        self.uiImageDirectoriesLabel = QtWidgets.QLabel(self.uiLocalBinaryImagePathsGroupBox)
        self.uiImageDirectoriesLabel.setObjectName("uiImageDirectoriesLabel")
        self.verticalLayout_10.addWidget(self.uiImageDirectoriesLabel)
        self.uiImageDirectoriesListWidget = QtWidgets.QListWidget(self.uiLocalBinaryImagePathsGroupBox)
        self.uiImageDirectoriesListWidget.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.uiImageDirectoriesListWidget.setLineWidth(0)
        self.uiImageDirectoriesListWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.uiImageDirectoriesListWidget.setAlternatingRowColors(False)
        self.uiImageDirectoriesListWidget.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.uiImageDirectoriesListWidget.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.uiImageDirectoriesListWidget.setProperty("isWrapping", False)
        self.uiImageDirectoriesListWidget.setResizeMode(QtWidgets.QListView.Adjust)
        self.uiImageDirectoriesListWidget.setLayoutMode(QtWidgets.QListView.Batched)
        self.uiImageDirectoriesListWidget.setGridSize(QtCore.QSize(0, 18))
        self.uiImageDirectoriesListWidget.setViewMode(QtWidgets.QListView.ListMode)
        self.uiImageDirectoriesListWidget.setUniformItemSizes(False)
        self.uiImageDirectoriesListWidget.setSelectionRectVisible(True)
        self.uiImageDirectoriesListWidget.setObjectName("uiImageDirectoriesListWidget")
        self.verticalLayout_10.addWidget(self.uiImageDirectoriesListWidget)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.uiImageDirectoriesAddPushButton = QtWidgets.QPushButton(self.uiLocalBinaryImagePathsGroupBox)
        self.uiImageDirectoriesAddPushButton.setObjectName("uiImageDirectoriesAddPushButton")
        self.horizontalLayout_13.addWidget(self.uiImageDirectoriesAddPushButton)
        self.uiImageDirectoriesDeletePushButton = QtWidgets.QPushButton(self.uiLocalBinaryImagePathsGroupBox)
        self.uiImageDirectoriesDeletePushButton.setObjectName("uiImageDirectoriesDeletePushButton")
        self.horizontalLayout_13.addWidget(self.uiImageDirectoriesDeletePushButton)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem2)
        self.verticalLayout_10.addLayout(self.horizontalLayout_13)
        self.verticalLayout_7.addWidget(self.uiLocalBinaryImagePathsGroupBox)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_7.addItem(spacerItem3)
        self.uiMiscTabWidget.addTab(self.uiImagesTab, "")
        self.uiConsoleTab = QtWidgets.QWidget()
        self.uiConsoleTab.setObjectName("uiConsoleTab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiConsoleTab)
        self.verticalLayout_3.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiTelnetConsoleSettingsGroupBox = QtWidgets.QGroupBox(self.uiConsoleTab)
        self.uiTelnetConsoleSettingsGroupBox.setObjectName("uiTelnetConsoleSettingsGroupBox")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.uiTelnetConsoleSettingsGroupBox)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.uiTelnetConsoleCommandLabel = QtWidgets.QLabel(self.uiTelnetConsoleSettingsGroupBox)
        self.uiTelnetConsoleCommandLabel.setObjectName("uiTelnetConsoleCommandLabel")
        self.verticalLayout_9.addWidget(self.uiTelnetConsoleCommandLabel)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.uiTelnetConsoleCommandLineEdit = QtWidgets.QLineEdit(self.uiTelnetConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTelnetConsoleCommandLineEdit.sizePolicy().hasHeightForWidth())
        self.uiTelnetConsoleCommandLineEdit.setSizePolicy(sizePolicy)
        self.uiTelnetConsoleCommandLineEdit.setReadOnly(True)
        self.uiTelnetConsoleCommandLineEdit.setObjectName("uiTelnetConsoleCommandLineEdit")
        self.horizontalLayout_9.addWidget(self.uiTelnetConsoleCommandLineEdit)
        self.uiTelnetConsolePreconfiguredCommandPushButton = QtWidgets.QPushButton(self.uiTelnetConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTelnetConsolePreconfiguredCommandPushButton.sizePolicy().hasHeightForWidth())
        self.uiTelnetConsolePreconfiguredCommandPushButton.setSizePolicy(sizePolicy)
        self.uiTelnetConsolePreconfiguredCommandPushButton.setObjectName("uiTelnetConsolePreconfiguredCommandPushButton")
        self.horizontalLayout_9.addWidget(self.uiTelnetConsolePreconfiguredCommandPushButton)
        self.verticalLayout_9.addLayout(self.horizontalLayout_9)
        self.verticalLayout_3.addWidget(self.uiTelnetConsoleSettingsGroupBox)
        self.uiConsoleMiscGroupBox = QtWidgets.QGroupBox(self.uiConsoleTab)
        self.uiConsoleMiscGroupBox.setObjectName("uiConsoleMiscGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiConsoleMiscGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiDelayConsoleAllSpinBox = QtWidgets.QSpinBox(self.uiConsoleMiscGroupBox)
        self.uiDelayConsoleAllSpinBox.setMaximum(10000)
        self.uiDelayConsoleAllSpinBox.setProperty("value", 500)
        self.uiDelayConsoleAllSpinBox.setObjectName("uiDelayConsoleAllSpinBox")
        self.gridLayout_7.addWidget(self.uiDelayConsoleAllSpinBox, 1, 0, 1, 1)
        self.uiSlowConsoleAllLabel = QtWidgets.QLabel(self.uiConsoleMiscGroupBox)
        self.uiSlowConsoleAllLabel.setObjectName("uiSlowConsoleAllLabel")
        self.gridLayout_7.addWidget(self.uiSlowConsoleAllLabel, 0, 0, 1, 1)
        self.verticalLayout_3.addWidget(self.uiConsoleMiscGroupBox)
        spacerItem4 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem4)
        self.uiMiscTabWidget.addTab(self.uiConsoleTab, "")
        self.uiVNCTab = QtWidgets.QWidget()
        self.uiVNCTab.setObjectName("uiVNCTab")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.uiVNCTab)
        self.verticalLayout_6.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.uiVNCConsoleSettingsGroupBox = QtWidgets.QGroupBox(self.uiVNCTab)
        self.uiVNCConsoleSettingsGroupBox.setObjectName("uiVNCConsoleSettingsGroupBox")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.uiVNCConsoleSettingsGroupBox)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.uiVNCConsoleCommandLabel = QtWidgets.QLabel(self.uiVNCConsoleSettingsGroupBox)
        self.uiVNCConsoleCommandLabel.setObjectName("uiVNCConsoleCommandLabel")
        self.verticalLayout_8.addWidget(self.uiVNCConsoleCommandLabel)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.uiVNCConsoleCommandLineEdit = QtWidgets.QLineEdit(self.uiVNCConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVNCConsoleCommandLineEdit.sizePolicy().hasHeightForWidth())
        self.uiVNCConsoleCommandLineEdit.setSizePolicy(sizePolicy)
        self.uiVNCConsoleCommandLineEdit.setReadOnly(True)
        self.uiVNCConsoleCommandLineEdit.setObjectName("uiVNCConsoleCommandLineEdit")
        self.horizontalLayout_8.addWidget(self.uiVNCConsoleCommandLineEdit)
        self.uiVNCConsolePreconfiguredCommandPushButton = QtWidgets.QPushButton(self.uiVNCConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVNCConsolePreconfiguredCommandPushButton.sizePolicy().hasHeightForWidth())
        self.uiVNCConsolePreconfiguredCommandPushButton.setSizePolicy(sizePolicy)
        self.uiVNCConsolePreconfiguredCommandPushButton.setObjectName("uiVNCConsolePreconfiguredCommandPushButton")
        self.horizontalLayout_8.addWidget(self.uiVNCConsolePreconfiguredCommandPushButton)
        self.verticalLayout_8.addLayout(self.horizontalLayout_8)
        spacerItem5 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem5)
        self.verticalLayout_6.addWidget(self.uiVNCConsoleSettingsGroupBox)
        self.uiMiscTabWidget.addTab(self.uiVNCTab, "")
        self.uiSPICETab = QtWidgets.QWidget()
        self.uiSPICETab.setObjectName("uiSPICETab")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiSPICETab)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiSPICEConsoleSettingsGroupBox = QtWidgets.QGroupBox(self.uiSPICETab)
        self.uiSPICEConsoleSettingsGroupBox.setObjectName("uiSPICEConsoleSettingsGroupBox")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.uiSPICEConsoleSettingsGroupBox)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.uiSPICEConsoleCommandLabel = QtWidgets.QLabel(self.uiSPICEConsoleSettingsGroupBox)
        self.uiSPICEConsoleCommandLabel.setObjectName("uiSPICEConsoleCommandLabel")
        self.verticalLayout_11.addWidget(self.uiSPICEConsoleCommandLabel)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.uiSPICEConsoleCommandLineEdit = QtWidgets.QLineEdit(self.uiSPICEConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSPICEConsoleCommandLineEdit.sizePolicy().hasHeightForWidth())
        self.uiSPICEConsoleCommandLineEdit.setSizePolicy(sizePolicy)
        self.uiSPICEConsoleCommandLineEdit.setReadOnly(True)
        self.uiSPICEConsoleCommandLineEdit.setObjectName("uiSPICEConsoleCommandLineEdit")
        self.horizontalLayout_10.addWidget(self.uiSPICEConsoleCommandLineEdit)
        self.uiSPICEConsolePreconfiguredCommandPushButton = QtWidgets.QPushButton(self.uiSPICEConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSPICEConsolePreconfiguredCommandPushButton.sizePolicy().hasHeightForWidth())
        self.uiSPICEConsolePreconfiguredCommandPushButton.setSizePolicy(sizePolicy)
        self.uiSPICEConsolePreconfiguredCommandPushButton.setObjectName("uiSPICEConsolePreconfiguredCommandPushButton")
        self.horizontalLayout_10.addWidget(self.uiSPICEConsolePreconfiguredCommandPushButton)
        self.verticalLayout_11.addLayout(self.horizontalLayout_10)
        spacerItem6 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_11.addItem(spacerItem6)
        self.gridLayout_2.addWidget(self.uiSPICEConsoleSettingsGroupBox, 0, 0, 1, 1)
        self.uiMiscTabWidget.addTab(self.uiSPICETab, "")
        self.uiSceneTab = QtWidgets.QWidget()
        self.uiSceneTab.setObjectName("uiSceneTab")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.uiSceneTab)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.uiSceneWidthLabel = QtWidgets.QLabel(self.uiSceneTab)
        self.uiSceneWidthLabel.setObjectName("uiSceneWidthLabel")
        self.gridLayout_3.addWidget(self.uiSceneWidthLabel, 0, 0, 1, 1)
        self.uiNotePreviewLabel = QtWidgets.QLabel(self.uiSceneTab)
        self.uiNotePreviewLabel.setObjectName("uiNotePreviewLabel")
        self.gridLayout_3.addWidget(self.uiNotePreviewLabel, 13, 0, 1, 1)
        self.uiShowInterfaceLabelsOnNewProject = QtWidgets.QCheckBox(self.uiSceneTab)
        self.uiShowInterfaceLabelsOnNewProject.setObjectName("uiShowInterfaceLabelsOnNewProject")
        self.gridLayout_3.addWidget(self.uiShowInterfaceLabelsOnNewProject, 6, 0, 1, 2)
        self.uiSceneHeightLabel = QtWidgets.QLabel(self.uiSceneTab)
        self.uiSceneHeightLabel.setObjectName("uiSceneHeightLabel")
        self.gridLayout_3.addWidget(self.uiSceneHeightLabel, 1, 0, 1, 1)
        self.uiDefaultNoteStylePlainTextEdit = QtWidgets.QPlainTextEdit(self.uiSceneTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDefaultNoteStylePlainTextEdit.sizePolicy().hasHeightForWidth())
        self.uiDefaultNoteStylePlainTextEdit.setSizePolicy(sizePolicy)
        self.uiDefaultNoteStylePlainTextEdit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.uiDefaultNoteStylePlainTextEdit.setFocusPolicy(QtCore.Qt.NoFocus)
        self.uiDefaultNoteStylePlainTextEdit.setReadOnly(True)
        self.uiDefaultNoteStylePlainTextEdit.setObjectName("uiDefaultNoteStylePlainTextEdit")
        self.gridLayout_3.addWidget(self.uiDefaultNoteStylePlainTextEdit, 14, 0, 1, 3)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.uiDefaultNoteFontPushButton = QtWidgets.QPushButton(self.uiSceneTab)
        self.uiDefaultNoteFontPushButton.setObjectName("uiDefaultNoteFontPushButton")
        self.horizontalLayout_14.addWidget(self.uiDefaultNoteFontPushButton)
        self.uiDefaultNoteColorPushButton = QtWidgets.QPushButton(self.uiSceneTab)
        self.uiDefaultNoteColorPushButton.setObjectName("uiDefaultNoteColorPushButton")
        self.horizontalLayout_14.addWidget(self.uiDefaultNoteColorPushButton)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem7)
        self.gridLayout_3.addLayout(self.horizontalLayout_14, 15, 0, 1, 3)
        self.uiLimitSizeNodeSymbolCheckBox = QtWidgets.QCheckBox(self.uiSceneTab)
        self.uiLimitSizeNodeSymbolCheckBox.setObjectName("uiLimitSizeNodeSymbolCheckBox")
        self.gridLayout_3.addWidget(self.uiLimitSizeNodeSymbolCheckBox, 9, 0, 1, 2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiDefaultLabelFontPushButton = QtWidgets.QPushButton(self.uiSceneTab)
        self.uiDefaultLabelFontPushButton.setObjectName("uiDefaultLabelFontPushButton")
        self.horizontalLayout_5.addWidget(self.uiDefaultLabelFontPushButton)
        self.uiDefaultLabelColorPushButton = QtWidgets.QPushButton(self.uiSceneTab)
        self.uiDefaultLabelColorPushButton.setObjectName("uiDefaultLabelColorPushButton")
        self.horizontalLayout_5.addWidget(self.uiDefaultLabelColorPushButton)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem8)
        self.gridLayout_3.addLayout(self.horizontalLayout_5, 12, 0, 1, 3)
        self.uiDefaultLabelStylePlainTextEdit = QtWidgets.QPlainTextEdit(self.uiSceneTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDefaultLabelStylePlainTextEdit.sizePolicy().hasHeightForWidth())
        self.uiDefaultLabelStylePlainTextEdit.setSizePolicy(sizePolicy)
        self.uiDefaultLabelStylePlainTextEdit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.uiDefaultLabelStylePlainTextEdit.setFocusPolicy(QtCore.Qt.NoFocus)
        self.uiDefaultLabelStylePlainTextEdit.setReadOnly(True)
        self.uiDefaultLabelStylePlainTextEdit.setObjectName("uiDefaultLabelStylePlainTextEdit")
        self.gridLayout_3.addWidget(self.uiDefaultLabelStylePlainTextEdit, 11, 0, 1, 3)
        self.uiSceneHeightSpinBox = QtWidgets.QSpinBox(self.uiSceneTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSceneHeightSpinBox.sizePolicy().hasHeightForWidth())
        self.uiSceneHeightSpinBox.setSizePolicy(sizePolicy)
        self.uiSceneHeightSpinBox.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.uiSceneHeightSpinBox.setMinimum(500)
        self.uiSceneHeightSpinBox.setMaximum(1000000)
        self.uiSceneHeightSpinBox.setSingleStep(100)
        self.uiSceneHeightSpinBox.setProperty("value", 1000)
        self.uiSceneHeightSpinBox.setObjectName("uiSceneHeightSpinBox")
        self.gridLayout_3.addWidget(self.uiSceneHeightSpinBox, 1, 1, 1, 1)
        self.uiSceneWidthSpinBox = QtWidgets.QSpinBox(self.uiSceneTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSceneWidthSpinBox.sizePolicy().hasHeightForWidth())
        self.uiSceneWidthSpinBox.setSizePolicy(sizePolicy)
        self.uiSceneWidthSpinBox.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.uiSceneWidthSpinBox.setMinimum(500)
        self.uiSceneWidthSpinBox.setMaximum(1000000)
        self.uiSceneWidthSpinBox.setSingleStep(100)
        self.uiSceneWidthSpinBox.setProperty("value", 2000)
        self.uiSceneWidthSpinBox.setObjectName("uiSceneWidthSpinBox")
        self.gridLayout_3.addWidget(self.uiSceneWidthSpinBox, 0, 1, 1, 1)
        self.uiSnapToGridOnNewProject = QtWidgets.QCheckBox(self.uiSceneTab)
        self.uiSnapToGridOnNewProject.setObjectName("uiSnapToGridOnNewProject")
        self.gridLayout_3.addWidget(self.uiSnapToGridOnNewProject, 8, 0, 1, 2)
        self.uiDrawingGridSizeLabel = QtWidgets.QLabel(self.uiSceneTab)
        self.uiDrawingGridSizeLabel.setObjectName("uiDrawingGridSizeLabel")
        self.gridLayout_3.addWidget(self.uiDrawingGridSizeLabel, 3, 0, 1, 1)
        self.uiLabelPreviewLabel = QtWidgets.QLabel(self.uiSceneTab)
        self.uiLabelPreviewLabel.setObjectName("uiLabelPreviewLabel")
        self.gridLayout_3.addWidget(self.uiLabelPreviewLabel, 10, 0, 1, 1)
        self.uiNodeGridSizeLabel = QtWidgets.QLabel(self.uiSceneTab)
        self.uiNodeGridSizeLabel.setObjectName("uiNodeGridSizeLabel")
        self.gridLayout_3.addWidget(self.uiNodeGridSizeLabel, 2, 0, 1, 1)
        self.uiDrawingGridSizeSpinBox = QtWidgets.QSpinBox(self.uiSceneTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDrawingGridSizeSpinBox.sizePolicy().hasHeightForWidth())
        self.uiDrawingGridSizeSpinBox.setSizePolicy(sizePolicy)
        self.uiDrawingGridSizeSpinBox.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.uiDrawingGridSizeSpinBox.setMinimum(5)
        self.uiDrawingGridSizeSpinBox.setMaximum(100)
        self.uiDrawingGridSizeSpinBox.setSingleStep(5)
        self.uiDrawingGridSizeSpinBox.setProperty("value", 25)
        self.uiDrawingGridSizeSpinBox.setObjectName("uiDrawingGridSizeSpinBox")
        self.gridLayout_3.addWidget(self.uiDrawingGridSizeSpinBox, 3, 1, 1, 1)
        self.uiNodeGridSizeSpinBox = QtWidgets.QSpinBox(self.uiSceneTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNodeGridSizeSpinBox.sizePolicy().hasHeightForWidth())
        self.uiNodeGridSizeSpinBox.setSizePolicy(sizePolicy)
        self.uiNodeGridSizeSpinBox.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.uiNodeGridSizeSpinBox.setMinimum(5)
        self.uiNodeGridSizeSpinBox.setMaximum(150)
        self.uiNodeGridSizeSpinBox.setSingleStep(5)
        self.uiNodeGridSizeSpinBox.setProperty("value", 75)
        self.uiNodeGridSizeSpinBox.setObjectName("uiNodeGridSizeSpinBox")
        self.gridLayout_3.addWidget(self.uiNodeGridSizeSpinBox, 2, 1, 1, 1)
        self.uiShowGridOnNewProject = QtWidgets.QCheckBox(self.uiSceneTab)
        self.uiShowGridOnNewProject.setObjectName("uiShowGridOnNewProject")
        self.gridLayout_3.addWidget(self.uiShowGridOnNewProject, 7, 0, 1, 2)
        self.uiRectangleSelectedItemCheckBox = QtWidgets.QCheckBox(self.uiSceneTab)
        self.uiRectangleSelectedItemCheckBox.setChecked(True)
        self.uiRectangleSelectedItemCheckBox.setObjectName("uiRectangleSelectedItemCheckBox")
        self.gridLayout_3.addWidget(self.uiRectangleSelectedItemCheckBox, 4, 0, 1, 3)
        spacerItem9 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem9, 16, 0, 1, 1)
        self.uiDrawLinkStatusPointsCheckBox = QtWidgets.QCheckBox(self.uiSceneTab)
        self.uiDrawLinkStatusPointsCheckBox.setChecked(True)
        self.uiDrawLinkStatusPointsCheckBox.setObjectName("uiDrawLinkStatusPointsCheckBox")
        self.gridLayout_3.addWidget(self.uiDrawLinkStatusPointsCheckBox, 5, 0, 1, 2)
        self.uiMiscTabWidget.addTab(self.uiSceneTab, "")
        self.uiMiscTab = QtWidgets.QWidget()
        self.uiMiscTab.setObjectName("uiMiscTab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiMiscTab)
        self.verticalLayout_2.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiCheckForUpdateCheckBox = QtWidgets.QCheckBox(self.uiMiscTab)
        self.uiCheckForUpdateCheckBox.setChecked(True)
        self.uiCheckForUpdateCheckBox.setObjectName("uiCheckForUpdateCheckBox")
        self.verticalLayout_2.addWidget(self.uiCheckForUpdateCheckBox)
        self.uiAutoOpenReadmeCheckBox = QtWidgets.QCheckBox(self.uiMiscTab)
        self.uiAutoOpenReadmeCheckBox.setChecked(True)
        self.uiAutoOpenReadmeCheckBox.setObjectName("uiAutoOpenReadmeCheckBox")
        self.verticalLayout_2.addWidget(self.uiAutoOpenReadmeCheckBox)
        self.uiCrashReportCheckBox = QtWidgets.QCheckBox(self.uiMiscTab)
        self.uiCrashReportCheckBox.setChecked(True)
        self.uiCrashReportCheckBox.setObjectName("uiCrashReportCheckBox")
        self.verticalLayout_2.addWidget(self.uiCrashReportCheckBox)
        self.uiOverlayNotificationsCheckBox = QtWidgets.QCheckBox(self.uiMiscTab)
        self.uiOverlayNotificationsCheckBox.setObjectName("uiOverlayNotificationsCheckBox")
        self.verticalLayout_2.addWidget(self.uiOverlayNotificationsCheckBox)
        self.uiExperimentalFeaturesCheckBox = QtWidgets.QCheckBox(self.uiMiscTab)
        self.uiExperimentalFeaturesCheckBox.setObjectName("uiExperimentalFeaturesCheckBox")
        self.verticalLayout_2.addWidget(self.uiExperimentalFeaturesCheckBox)
        self.uiHdpiCheckBox = QtWidgets.QCheckBox(self.uiMiscTab)
        self.uiHdpiCheckBox.setObjectName("uiHdpiCheckBox")
        self.verticalLayout_2.addWidget(self.uiHdpiCheckBox)
        self.uiMultiProfilesCheckBox = QtWidgets.QCheckBox(self.uiMiscTab)
        self.uiMultiProfilesCheckBox.setObjectName("uiMultiProfilesCheckBox")
        self.verticalLayout_2.addWidget(self.uiMultiProfilesCheckBox)
        spacerItem10 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem10)
        self.uiMiscTabWidget.addTab(self.uiMiscTab, "")
        self.verticalLayout.addWidget(self.uiMiscTabWidget)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        spacerItem11 = QtWidgets.QSpacerItem(324, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem11)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(GeneralPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_6.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_6)

        self.retranslateUi(GeneralPreferencesPageWidget)
        self.uiMiscTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(GeneralPreferencesPageWidget)
        GeneralPreferencesPageWidget.setTabOrder(self.uiProjectsPathLineEdit, self.uiProjectsPathToolButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiProjectsPathToolButton, self.uiSymbolsPathLineEdit)
        GeneralPreferencesPageWidget.setTabOrder(self.uiSymbolsPathLineEdit, self.uiSymbolsPathToolButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiSymbolsPathToolButton, self.uiConfigsPathLineEdit)
        GeneralPreferencesPageWidget.setTabOrder(self.uiConfigsPathLineEdit, self.uiConfigsPathToolButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiConfigsPathToolButton, self.uiAppliancesPathLineEdit)
        GeneralPreferencesPageWidget.setTabOrder(self.uiAppliancesPathLineEdit, self.uiAppliancesPathToolButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiAppliancesPathToolButton, self.uiStyleComboBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiStyleComboBox, self.uiSymbolThemeComboBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiSymbolThemeComboBox, self.uiImportConfigurationFilePushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiImportConfigurationFilePushButton, self.uiExportConfigurationFilePushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiExportConfigurationFilePushButton, self.uiBrowseConfigurationPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiBrowseConfigurationPushButton, self.uiImagesPathLineEdit)
        GeneralPreferencesPageWidget.setTabOrder(self.uiImagesPathLineEdit, self.uiImagesPathToolButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiImagesPathToolButton, self.uiImageDirectoriesAddPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiImageDirectoriesAddPushButton, self.uiImageDirectoriesDeletePushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiImageDirectoriesDeletePushButton, self.uiTelnetConsoleCommandLineEdit)
        GeneralPreferencesPageWidget.setTabOrder(self.uiTelnetConsoleCommandLineEdit, self.uiTelnetConsolePreconfiguredCommandPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiTelnetConsolePreconfiguredCommandPushButton, self.uiDelayConsoleAllSpinBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiDelayConsoleAllSpinBox, self.uiVNCConsoleCommandLineEdit)
        GeneralPreferencesPageWidget.setTabOrder(self.uiVNCConsoleCommandLineEdit, self.uiVNCConsolePreconfiguredCommandPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiVNCConsolePreconfiguredCommandPushButton, self.uiSPICEConsoleCommandLineEdit)
        GeneralPreferencesPageWidget.setTabOrder(self.uiSPICEConsoleCommandLineEdit, self.uiSPICEConsolePreconfiguredCommandPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiSPICEConsolePreconfiguredCommandPushButton, self.uiSceneWidthSpinBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiSceneWidthSpinBox, self.uiSceneHeightSpinBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiSceneHeightSpinBox, self.uiNodeGridSizeSpinBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiNodeGridSizeSpinBox, self.uiDrawingGridSizeSpinBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiDrawingGridSizeSpinBox, self.uiRectangleSelectedItemCheckBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiRectangleSelectedItemCheckBox, self.uiDrawLinkStatusPointsCheckBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiDrawLinkStatusPointsCheckBox, self.uiShowInterfaceLabelsOnNewProject)
        GeneralPreferencesPageWidget.setTabOrder(self.uiShowInterfaceLabelsOnNewProject, self.uiShowGridOnNewProject)
        GeneralPreferencesPageWidget.setTabOrder(self.uiShowGridOnNewProject, self.uiSnapToGridOnNewProject)
        GeneralPreferencesPageWidget.setTabOrder(self.uiSnapToGridOnNewProject, self.uiLimitSizeNodeSymbolCheckBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiLimitSizeNodeSymbolCheckBox, self.uiDefaultLabelFontPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiDefaultLabelFontPushButton, self.uiDefaultLabelColorPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiDefaultLabelColorPushButton, self.uiDefaultNoteFontPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiDefaultNoteFontPushButton, self.uiDefaultNoteColorPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiDefaultNoteColorPushButton, self.uiCheckForUpdateCheckBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiCheckForUpdateCheckBox, self.uiCrashReportCheckBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiCrashReportCheckBox, self.uiOverlayNotificationsCheckBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiOverlayNotificationsCheckBox, self.uiExperimentalFeaturesCheckBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiExperimentalFeaturesCheckBox, self.uiHdpiCheckBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiHdpiCheckBox, self.uiMultiProfilesCheckBox)
        GeneralPreferencesPageWidget.setTabOrder(self.uiMultiProfilesCheckBox, self.uiRestoreDefaultsPushButton)
        GeneralPreferencesPageWidget.setTabOrder(self.uiRestoreDefaultsPushButton, self.uiMiscTabWidget)

    def retranslateUi(self, GeneralPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        GeneralPreferencesPageWidget.setWindowTitle(_translate("GeneralPreferencesPageWidget", "General"))
        self.uiLocalPathsGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Local paths"))
        self.uiProjectsPathLabel.setText(_translate("GeneralPreferencesPageWidget", "My projects:"))
        self.uiProjectsPathLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "Directory where your GNS3 projects are stored"))
        self.uiProjectsPathToolButton.setText(_translate("GeneralPreferencesPageWidget", "&Browse..."))
        self.label.setText(_translate("GeneralPreferencesPageWidget", "My symbols:"))
        self.uiSymbolsPathToolButton.setText(_translate("GeneralPreferencesPageWidget", "Browse..."))
        self.uiConfigsPathLabel.setText(_translate("GeneralPreferencesPageWidget", "My configs:"))
        self.uiConfigsPathLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "Directory where your binary images (e.g. IOS) are stored"))
        self.uiConfigsPathToolButton.setText(_translate("GeneralPreferencesPageWidget", "&Browse..."))
        self.label_3.setText(_translate("GeneralPreferencesPageWidget", "My custom appliances:"))
        self.uiAppliancesPathToolButton.setText(_translate("GeneralPreferencesPageWidget", "Browse..."))
        self.uiStyleGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Interface style"))
        self.uiSymbolThemeGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Symbol theme for new templates"))
        self.uiSymbolThemeComboBox.setToolTip(_translate("GeneralPreferencesPageWidget", "Symbol theme support only works when adding a new template using the recommended method in the template wizard."))
        self.uiConfigurationFileGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Configuration file"))
        self.uiImportConfigurationFilePushButton.setText(_translate("GeneralPreferencesPageWidget", "&Import"))
        self.uiExportConfigurationFilePushButton.setText(_translate("GeneralPreferencesPageWidget", "&Export"))
        self.uiBrowseConfigurationPushButton.setText(_translate("GeneralPreferencesPageWidget", "Browse configuration directory"))
        self.uiConfigurationFileLabel.setText(_translate("GeneralPreferencesPageWidget", "Unknown location"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiGeneralTab), _translate("GeneralPreferencesPageWidget", "General"))
        self.uiLocalBinaryImagePathsGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Local paths"))
        self.uiImagesPathLabel.setText(_translate("GeneralPreferencesPageWidget", "My binary images:"))
        self.uiImagesPathLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "Directory where your binary images (e.g. IOS) are stored"))
        self.uiImagesPathToolButton.setText(_translate("GeneralPreferencesPageWidget", "&Browse..."))
        self.uiImageDirectoriesLabel.setText(_translate("GeneralPreferencesPageWidget", "Additional search locations for binary images:"))
        self.uiImageDirectoriesAddPushButton.setText(_translate("GeneralPreferencesPageWidget", "Add"))
        self.uiImageDirectoriesDeletePushButton.setText(_translate("GeneralPreferencesPageWidget", "Delete"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiImagesTab), _translate("GeneralPreferencesPageWidget", "Binary images"))
        self.uiTelnetConsoleSettingsGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Console settings"))
        self.uiTelnetConsoleCommandLabel.setText(_translate("GeneralPreferencesPageWidget", "Console application command for Telnet:"))
        self.uiTelnetConsoleCommandLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "<html><head/><body><p>Command line replacements:</p><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%h or {host} = console IP or hostname</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%p or {port} = console port</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%d or {name} = title of the console (node name)</li><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%P or {project} = project</li></ul><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%i or {project_id} = project UUID</li><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%n or {node_id} = node UUID</li></ul><li style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%c or {url} = server URL</li></ul></body></html>"))
        self.uiTelnetConsolePreconfiguredCommandPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Edit"))
        self.uiConsoleMiscGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Miscellaneous"))
        self.uiDelayConsoleAllSpinBox.setSuffix(_translate("GeneralPreferencesPageWidget", " ms"))
        self.uiSlowConsoleAllLabel.setText(_translate("GeneralPreferencesPageWidget", "Delay between each console launch when consoling to all devices:"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiConsoleTab), _translate("GeneralPreferencesPageWidget", "Console applications"))
        self.uiVNCConsoleSettingsGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Settings for VNC connections"))
        self.uiVNCConsoleCommandLabel.setText(_translate("GeneralPreferencesPageWidget", "Console application command for VNC:"))
        self.uiVNCConsoleCommandLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "<html><head/><body><p>Command line replacements:</p><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%h or {host} = console IP or hostname</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%p or {port} = console port</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%P or {display} = VNC display</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%d or {name} = node name</li><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%T or {project} = project name</li></ul><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%i or {project_id} = project UUID</li><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%n or {node_id} = node UUID</li></ul><li style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%c or {url} = server URL</li></ul></body></html>"))
        self.uiVNCConsolePreconfiguredCommandPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Edit"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiVNCTab), _translate("GeneralPreferencesPageWidget", "VNC"))
        self.uiSPICEConsoleSettingsGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Settings for SPICE connections"))
        self.uiSPICEConsoleCommandLabel.setText(_translate("GeneralPreferencesPageWidget", "Console application command for SPICE:"))
        self.uiSPICEConsoleCommandLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "<html><head/><body><p>Command line replacements:</p><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%h or {host} = console IP or hostname</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%p or {port} = console port</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%d or {name} = node name</li><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%P or {project} = project name</li></ul><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%i or {project_id} = project UUID</li><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%n or {node_id} = node UUID</li></ul><li style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">%c or {url} = server URL</li></ul></body></html>"))
        self.uiSPICEConsolePreconfiguredCommandPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Edit"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiSPICETab), _translate("GeneralPreferencesPageWidget", "SPICE"))
        self.uiSceneWidthLabel.setText(_translate("GeneralPreferencesPageWidget", "Default width:"))
        self.uiNotePreviewLabel.setText(_translate("GeneralPreferencesPageWidget", "Default note style:"))
        self.uiShowInterfaceLabelsOnNewProject.setText(_translate("GeneralPreferencesPageWidget", "Show interface labels on new project"))
        self.uiSceneHeightLabel.setText(_translate("GeneralPreferencesPageWidget", "Default height:"))
        self.uiDefaultNoteStylePlainTextEdit.setPlainText(_translate("GeneralPreferencesPageWidget", "AaBbYyZz"))
        self.uiDefaultNoteFontPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Select default font"))
        self.uiDefaultNoteColorPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Select default color"))
        self.uiLimitSizeNodeSymbolCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Limit the size of node symbols"))
        self.uiDefaultLabelFontPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Select default font"))
        self.uiDefaultLabelColorPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Select default color"))
        self.uiDefaultLabelStylePlainTextEdit.setPlainText(_translate("GeneralPreferencesPageWidget", "AaBbYyZz"))
        self.uiSceneHeightSpinBox.setSuffix(_translate("GeneralPreferencesPageWidget", " pixels"))
        self.uiSceneWidthSpinBox.setSuffix(_translate("GeneralPreferencesPageWidget", " pixels"))
        self.uiSnapToGridOnNewProject.setText(_translate("GeneralPreferencesPageWidget", "Snap to grid on new project"))
        self.uiDrawingGridSizeLabel.setText(_translate("GeneralPreferencesPageWidget", "Default drawing grid size:"))
        self.uiLabelPreviewLabel.setText(_translate("GeneralPreferencesPageWidget", "Default label style:"))
        self.uiNodeGridSizeLabel.setText(_translate("GeneralPreferencesPageWidget", "Default node grid size:"))
        self.uiShowGridOnNewProject.setText(_translate("GeneralPreferencesPageWidget", "Show grid on new project"))
        self.uiRectangleSelectedItemCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Draw a rectangle when an item is selected"))
        self.uiDrawLinkStatusPointsCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Draw link status points"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiSceneTab), _translate("GeneralPreferencesPageWidget", "Topology view"))
        self.uiCheckForUpdateCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Automatically check for update"))
        self.uiAutoOpenReadmeCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Automatically open project README files"))
        self.uiCrashReportCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Send anonymous crash reports"))
        self.uiOverlayNotificationsCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Display error, warning and info in an overlay popup"))
        self.uiExperimentalFeaturesCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Enable experimental features"))
        self.uiHdpiCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Enable HDPI mode (this may crash on Linux, restart required)"))
        self.uiMultiProfilesCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Request for profile settings at application startup"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiMiscTab), _translate("GeneralPreferencesPageWidget", "Miscellaneous"))
        self.uiRestoreDefaultsPushButton.setText(_translate("GeneralPreferencesPageWidget", "Restore defaults"))
